@@ -3,6 +3,7 @@
 #ifndef RGBDS_ASM_CHARMAP_HPP
 #define RGBDS_ASM_CHARMAP_HPP
 
+#include <optional>
 #include <stdint.h>
 #include <string>
 #include <string_view>
@@ -22,6 +23,7 @@ void charmap_CheckStack();
 void charmap_Add(std::string const &mapping, std::vector<int32_t> &&value);
 bool charmap_HasChar(std::string const &mapping);
 size_t charmap_CharSize(std::string const &mapping);
+std::optional<int32_t> charmap_CharValue(std::string const &mapping, size_t idx);
 std::vector<int32_t> charmap_Convert(std::string const &input);
 size_t charmap_ConvertNext(std::string_view &input, std::vector<int32_t> *output);
 std::string charmap_Reverse(std::vector<int32_t> const &value, bool &unique);
