@@ -5,9 +5,6 @@
 
 #include <stdint.h>
 
-extern uint8_t fixPrecision;
-
-uint8_t fix_Precision();
 int32_t fix_Sin(int32_t i, int32_t q);
 int32_t fix_Cos(int32_t i, int32_t q);
 int32_t fix_Tan(int32_t i, int32_t q);
