@@ -3,7 +3,16 @@
 #ifndef RGBDS_ASM_WARNING_HPP
 #define RGBDS_ASM_WARNING_HPP
 
-extern unsigned int nbErrors, maxErrors;
+#include <functional>
+
+#include "diagnostics.hpp"
+
+enum WarningLevel {
+	LEVEL_DEFAULT,    // Warnings that are enabled by default
+	LEVEL_ALL,        // Warnings that probably indicate an error
+	LEVEL_EXTRA,      // Warnings that are less likely to indicate an error
+	LEVEL_EVERYTHING, // Literally every warning
+};
 
 enum WarningID {
 	WARNING_ASSERT,               // Assertions
@@ -43,24 +52,7 @@ enum WarningID {
 	NB_WARNINGS,
 };
 
-enum WarningAbled { WARNING_DEFAULT, WARNING_ENABLED, WARNING_DISABLED };
-
-struct WarningState {
-	WarningAbled state;
-	WarningAbled error;
-
-	void update(WarningState other);
-};
-
-struct Diagnostics {
-	WarningState flagStates[NB_WARNINGS];
-	WarningState metaStates[NB_WARNINGS];
-};
-
-extern Diagnostics warningStates;
-extern bool warningsAreErrors;
-
-void processWarningFlag(char const *flag);
+extern Diagnostics<WarningLevel, WarningID> warnings;
 
 // Used to warn the user about problems that don't prevent the generation of
 // valid code.
@@ -73,7 +65,7 @@ void warning(WarningID id, char const *fmt, ...);
 // It is also used when the assembler goes into an invalid state (for example,
 // when it fails to allocate memory).
 [[gnu::format(printf, 1, 2), noreturn]]
-void fatalerror(char const *fmt, ...);
+void fatal(char const *fmt, ...);
 
 // Used for errors that make it impossible to assemble correctly, but don't
 // affect the following code. The code will fail to assemble but the user will
@@ -82,4 +74,12 @@ void fatalerror(char const *fmt, ...);
 [[gnu::format(printf, 1, 2)]]
 void error(char const *fmt, ...);
 
+// Used for errors that make it impossible to assemble correctly, but don't
+// affect the following code. The code will fail to assemble but the user will
+// get a list of all errors at the end, making it easier to fix all of them at
+// once.
+void error(std::function<void()> callback);
+
+void requireZeroErrors();
+
 #endif // RGBDS_ASM_WARNING_HPP
