@@ -40,7 +40,10 @@ struct Options {
 		uint16_t top;
 		uint16_t width;
 		uint16_t height;
+		uint32_t right() const { return left + width * 8; }
+		uint32_t bottom() const { return top + height * 8; }
 	} inputSlice{0, 0, 0, 0};                          // -L (margins in clockwise order, like CSS)
+	uint8_t basePalID = 0;                             // -l
 	std::array<uint16_t, 2> maxNbTiles{UINT16_MAX, 0}; // -N
 	uint16_t nbPalettes = 8;                           // -n
 	std::string output{};                              // -o
@@ -68,6 +71,8 @@ struct Options {
 	mutable bool hasTransparentPixels = false;
 	uint8_t maxOpaqueColors() const { return nbColorsPerPal - hasTransparentPixels; }
 
+	uint16_t maxNbColors() const { return nbColorsPerPal * nbPalettes; }
+
 	uint8_t dmgColors[4] = {};
 	uint8_t dmgValue(uint8_t i) const {
 		assume(i < 4);
@@ -77,25 +82,6 @@ struct Options {
 
 extern Options options;
 
-// Prints the error count, and exits with failure
-[[noreturn]]
-void giveUp();
-// If any error has been emitted thus far, calls `giveUp()`.
-void requireZeroErrors();
-// Prints a warning, and does not change the error count
-[[gnu::format(printf, 1, 2)]]
-void warning(char const *fmt, ...);
-// Prints an error, and increments the error count
-[[gnu::format(printf, 1, 2)]]
-void error(char const *fmt, ...);
-// Prints an error, and increments the error count
-// Does not take format arguments so `format_` and `-Wformat-security` won't complain about
-// calling `errorMessage(msg)`.
-void errorMessage(char const *msg);
-// Prints a fatal error, increments the error count, and gives up
-[[gnu::format(printf, 1, 2), noreturn]]
-void fatal(char const *fmt, ...);
-
 struct Palette {
 	// An array of 4 GBC-native (RGB555) colors
 	std::array<uint16_t, 4> colors{UINT16_MAX, UINT16_MAX, UINT16_MAX, UINT16_MAX};
@@ -114,9 +100,9 @@ struct Palette {
 };
 
 // Flipping tends to happen fairly often, so take a bite out of dcache to speed it up
-static constexpr auto flipTable = ([]() constexpr {
+static std::array<uint16_t, 256> flipTable = ([]() constexpr {
 	std::array<uint16_t, 256> table{};
-	for (uint16_t i = 0; i < table.size(); i++) {
+	for (uint16_t i = 0; i < table.size(); ++i) {
 		// To flip all the bits, we'll flip both nibbles, then each nibble half, etc.
 		uint16_t byte = i;
 		byte = (byte & 0b0000'1111) << 4 | (byte & 0b1111'0000) >> 4;
