@@ -24,7 +24,7 @@ static inline void unreachable_() {
 	#ifdef _MSC_VER
 		#define assume(x) __assume(x)
 	#else
-		//  `[[gnu::assume()]]` for GCC or compatible also has insufficient support (GCC 13+ only)
+		// `[[gnu::assume()]]` for GCC or compatible also has insufficient support (GCC 13+ only)
 		#define assume(x) \
 			do { \
 				if (!(x)) { \
@@ -71,7 +71,7 @@ static inline int ctz(unsigned int x) {
 
 	while (!(x & 1)) {
 		x >>= 1;
-		cnt++;
+		++cnt;
 	}
 	return cnt;
 }
@@ -81,7 +81,7 @@ static inline int clz(unsigned int x) {
 
 	while (x <= UINT_MAX / 2) {
 		x <<= 1;
-		cnt++;
+		++cnt;
 	}
 	return cnt;
 }
