@@ -31,36 +31,34 @@ struct CharmapNode {
 struct Charmap {
 	std::string name;
 	std::vector<CharmapNode> nodes; // first node is reserved for the root node
+};
 
-	// Traverse the trie depth-first to derive the character mappings in definition order
-	template<typename F>
-	bool forEachChar(F callback) const {
-		// clang-format off: nested initializers
-		for (std::stack<std::pair<size_t, std::string>> prefixes({{0, ""}}); !prefixes.empty();) {
-			// clang-format on
-			auto [nodeIdx, mapping] = std::move(prefixes.top());
-			prefixes.pop();
-			CharmapNode const &node = nodes[nodeIdx];
-			if (node.isTerminal()) {
-				if (!callback(nodeIdx, mapping)) {
-					return false;
-				}
-			}
-			for (unsigned c = 0; c < std::size(node.next); c++) {
-				if (size_t nextIdx = node.next[c]; nextIdx) {
-					prefixes.push({nextIdx, mapping + static_cast<char>(c)});
-				}
+// Traverse the trie depth-first to derive the character mappings in definition order
+template<typename F>
+bool forEachChar(Charmap const &charmap, F callback) {
+	// clang-format off: nested initializers
+	for (std::stack<std::pair<size_t, std::string>> prefixes({{0, ""}}); !prefixes.empty();) {
+		// clang-format on
+		auto [nodeIdx, mapping] = std::move(prefixes.top());
+		prefixes.pop();
+		CharmapNode const &node = charmap.nodes[nodeIdx];
+		if (node.isTerminal() && !callback(nodeIdx, mapping)) {
+			return false;
+		}
+		for (unsigned c = 0; c < std::size(node.next); ++c) {
+			if (size_t nextIdx = node.next[c]; nextIdx) {
+				prefixes.push({nextIdx, mapping + static_cast<char>(c)});
 			}
 		}
-		return true;
 	}
-};
+	return true;
+}
 
 static std::deque<Charmap> charmapList;
 static std::unordered_map<std::string, size_t> charmapMap; // Indexes into `charmapList`
 
 static Charmap *currentCharmap;
-std::stack<Charmap *> charmapStack;
+static std::stack<Charmap *> charmapStack;
 
 bool charmap_ForEach(
     void (*mapFunc)(std::string const &),
@@ -68,7 +66,7 @@ bool charmap_ForEach(
 ) {
 	for (Charmap const &charmap : charmapList) {
 		std::map<size_t, std::string> mappings;
-		charmap.forEachChar([&mappings](size_t nodeIdx, std::string const &mapping) {
+		forEachChar(charmap, [&mappings](size_t nodeIdx, std::string const &mapping) {
 			mappings[nodeIdx] = mapping;
 			return true;
 		});
@@ -86,14 +84,14 @@ void charmap_New(std::string const &name, std::string const *baseName) {
 
 	if (baseName != nullptr) {
 		if (auto search = charmapMap.find(*baseName); search == charmapMap.end()) {
-			error("Base charmap '%s' doesn't exist\n", baseName->c_str());
+			error("Base charmap '%s' doesn't exist", baseName->c_str());
 		} else {
 			baseIdx = search->second;
 		}
 	}
 
 	if (charmapMap.find(name) != charmapMap.end()) {
-		error("Charmap '%s' already exists\n", name.c_str());
+		error("Charmap '%s' already exists", name.c_str());
 		return;
 	}
 
@@ -114,7 +112,7 @@ void charmap_New(std::string const &name, std::string const *baseName) {
 
 void charmap_Set(std::string const &name) {
 	if (auto search = charmapMap.find(name); search == charmapMap.end()) {
-		error("Charmap '%s' doesn't exist\n", name.c_str());
+		error("Charmap '%s' doesn't exist", name.c_str());
 	} else {
 		currentCharmap = &charmapList[search->second];
 	}
@@ -126,7 +124,7 @@ void charmap_Push() {
 
 void charmap_Pop() {
 	if (charmapStack.empty()) {
-		error("No entries in the charmap stack\n");
+		error("No entries in the charmap stack");
 		return;
 	}
 
@@ -136,13 +134,13 @@ void charmap_Pop() {
 
 void charmap_CheckStack() {
 	if (!charmapStack.empty()) {
-		warning(WARNING_UNMATCHED_DIRECTIVE, "`PUSHC` without corresponding `POPC`\n");
+		warning(WARNING_UNMATCHED_DIRECTIVE, "`PUSHC` without corresponding `POPC`");
 	}
 }
 
 void charmap_Add(std::string const &mapping, std::vector<int32_t> &&value) {
 	if (mapping.empty()) {
-		error("Cannot map an empty string\n");
+		error("Cannot map an empty string");
 		return;
 	}
 
@@ -168,7 +166,7 @@ void charmap_Add(std::string const &mapping, std::vector<int32_t> &&value) {
 	CharmapNode &node = charmap.nodes[nodeIdx];
 
 	if (node.isTerminal()) {
-		warning(WARNING_CHARMAP_REDEF, "Overriding charmap mapping\n");
+		warning(WARNING_CHARMAP_REDEF, "Overriding charmap mapping");
 	}
 
 	std::swap(node.value, value);
@@ -240,13 +238,13 @@ size_t charmap_ConvertNext(std::string_view &input, std::vector<int32_t> *output
 			break;
 		}
 
-		inputIdx++; // Consume that char
+		++inputIdx; // Consume that char
 
 		if (charmap.nodes[nodeIdx].isTerminal()) {
 			matchIdx = nodeIdx; // This node matches, register it
 			rewindDistance = 0; // If no longer match is found, rewind here
 		} else {
-			rewindDistance++;
+			++rewindDistance;
 		}
 	}
 
@@ -266,14 +264,15 @@ size_t charmap_ConvertNext(std::string_view &input, std::vector<int32_t> *output
 	} else if (inputIdx < input.length()) { // No match found, but there is some input left
 		size_t codepointLen = 0;
 		// This will write the codepoint's value to `output`, little-endian
-		for (uint32_t state = 0, codepoint = 0; inputIdx + codepointLen < input.length();) {
-			if (decode(&state, &codepoint, input[inputIdx + codepointLen]) == 1) {
-				error("Input string is not valid UTF-8\n");
+		for (uint32_t state = UTF8_ACCEPT, codepoint = 0;
+		     inputIdx + codepointLen < input.length();) {
+			if (decode(&state, &codepoint, input[inputIdx + codepointLen]) == UTF8_REJECT) {
+				error("Input string is not valid UTF-8");
 				codepointLen = 1;
 				break;
 			}
-			codepointLen++;
-			if (state == 0) {
+			++codepointLen;
+			if (state == UTF8_ACCEPT) {
 				break;
 			}
 		}
@@ -286,11 +285,11 @@ size_t charmap_ConvertNext(std::string_view &input, std::vector<int32_t> *output
 
 		// Warn if this character is not mapped but any others are
 		if (int firstChar = input[inputIdx]; charmap.nodes.size() > 1) {
-			warning(WARNING_UNMAPPED_CHAR_1, "Unmapped character %s\n", printChar(firstChar));
+			warning(WARNING_UNMAPPED_CHAR_1, "Unmapped character %s", printChar(firstChar));
 		} else if (charmap.name != DEFAULT_CHARMAP_NAME) {
 			warning(
 			    WARNING_UNMAPPED_CHAR_2,
-			    "Unmapped character %s not in " DEFAULT_CHARMAP_NAME " charmap\n",
+			    "Unmapped character %s not in " DEFAULT_CHARMAP_NAME " charmap",
 			    printChar(firstChar)
 			);
 		}
@@ -306,7 +305,7 @@ size_t charmap_ConvertNext(std::string_view &input, std::vector<int32_t> *output
 std::string charmap_Reverse(std::vector<int32_t> const &value, bool &unique) {
 	Charmap const &charmap = *currentCharmap;
 	std::string revMapping;
-	unique = charmap.forEachChar([&](size_t nodeIdx, std::string const &mapping) {
+	unique = forEachChar(charmap, [&](size_t nodeIdx, std::string const &mapping) {
 		if (charmap.nodes[nodeIdx].value == value) {
 			if (revMapping.empty()) {
 				revMapping = mapping;
