@@ -3,6 +3,7 @@
 #include "asm/fstack.hpp"
 #include <sys/stat.h>
 
+#include <deque>
 #include <errno.h>
 #include <inttypes.h>
 #include <memory>
@@ -10,7 +11,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 
-#include "error.hpp"
+#include "diagnostics.hpp"
 #include "helpers.hpp"
 #include "linkdefs.hpp"
 #include "platform.hpp" // S_ISDIR (stat macro)
@@ -41,23 +42,29 @@ struct Context {
 };
 
 static std::stack<Context> contextStack;
-size_t maxRecursionDepth;
 
 // The first include path for `fstk_FindFile` to try is none at all
-static std::vector<std::string> includePaths = {""};
+static std::vector<std::string> includePaths = {""}; // -I
+static std::deque<std::string> preIncludeNames;      // -P
+static bool failedOnMissingInclude = false;
 
-static std::string preIncludeName;
+std::string FileStackNode::reptChain() const {
+	std::string chain;
+	std::vector<uint32_t> const &nodeIters = iters();
+	for (uint32_t i = nodeIters.size(); i--;) {
+		chain.append("::REPT~");
+		chain.append(std::to_string(nodeIters[i]));
+	}
+	return chain;
+}
 
 std::string const &FileStackNode::dump(uint32_t curLineNo) const {
-	if (data.holds<std::vector<uint32_t>>()) {
+	if (std::holds_alternative<std::vector<uint32_t>>(data)) {
 		assume(parent); // REPT nodes use their parent's name
 		std::string const &lastName = parent->dump(lineNo);
 		fputs(" -> ", stderr);
 		fputs(lastName.c_str(), stderr);
-		std::vector<uint32_t> const &nodeIters = iters();
-		for (uint32_t i = nodeIters.size(); i--;) {
-			fprintf(stderr, "::REPT~%" PRIu32, nodeIters[i]);
-		}
+		fputs(reptChain().c_str(), stderr);
 		fprintf(stderr, "(%" PRIu32 ")", curLineNo);
 		return lastName;
 	} else {
@@ -72,12 +79,13 @@ std::string const &FileStackNode::dump(uint32_t curLineNo) const {
 	}
 }
 
-void fstk_DumpCurrent() {
+bool fstk_DumpCurrent() {
 	if (lexer_AtTopLevel()) {
-		fputs("at top level", stderr);
-		return;
+		return false;
 	}
+	assume(!contextStack.empty());
 	contextStack.top().fileInfo->dump(lexer_GetLineNo());
+	return true;
 }
 
 std::shared_ptr<FileStackNode> fstk_GetFileStack() {
@@ -114,16 +122,9 @@ void fstk_AddIncludePath(std::string const &path) {
 	}
 }
 
-void fstk_SetPreIncludeFile(std::string const &path) {
-	if (!preIncludeName.empty()) {
-		warnx("Overriding pre-included filename %s", preIncludeName.c_str());
-	}
-	preIncludeName = path;
-	// LCOV_EXCL_START
-	if (verbose) {
-		printf("Pre-included filename %s\n", preIncludeName.c_str());
-	}
-	// LCOV_EXCL_STOP
+void fstk_AddPreIncludeFile(std::string const &path) {
+	preIncludeNames.emplace_front(path);
+	verbosePrint("Pre-included filename %s\n", path.c_str()); // LCOV_EXCL_LINE
 }
 
 static bool isValidFilePath(std::string const &path) {
@@ -132,11 +133,9 @@ static bool isValidFilePath(std::string const &path) {
 }
 
 static void printDep(std::string const &path) {
-	if (dependFile) {
-		fprintf(dependFile, "%s: %s\n", targetFileName.c_str(), path.c_str());
-		if (generatePhonyDeps && isValidFilePath(path)) {
-			fprintf(dependFile, "%s:\n", path.c_str());
-		}
+	options.printDep(path);
+	if (options.dependFile && options.generatePhonyDeps && isValidFilePath(path)) {
+		fprintf(options.dependFile, "%s:\n", path.c_str());
 	}
 }
 
@@ -149,7 +148,7 @@ std::optional<std::string> fstk_FindFile(std::string const &path) {
 	}
 
 	errno = ENOENT;
-	if (generatedMissingIncludes) {
+	if (options.missingIncludeState != INC_ERROR) {
 		printDep(path);
 	}
 	return std::nullopt;
@@ -159,8 +158,8 @@ bool yywrap() {
 	uint32_t ifDepth = lexer_GetIFDepth();
 
 	if (ifDepth != 0) {
-		fatalerror(
-		    "Ended block with %" PRIu32 " unterminated IF construct%s\n",
+		fatal(
+		    "Ended block with %" PRIu32 " unterminated IF construct%s",
 		    ifDepth,
 		    ifDepth == 1 ? "" : "s"
 		);
@@ -188,11 +187,11 @@ bool yywrap() {
 
 			// This error message will refer to the current iteration
 			if (sym->type != SYM_VAR) {
-				fatalerror("Failed to update FOR symbol value\n");
+				fatal("Failed to update FOR symbol value");
 			}
 		}
 		// Advance to the next iteration
-		fileInfoIters.front()++;
+		++fileInfoIters.front();
 		// If this wasn't the last iteration, wrap instead of popping
 		if (fileInfoIters.front() <= context.nbReptIters) {
 			lexer_RestartRept(context.fileInfo->lineNo);
@@ -210,12 +209,12 @@ bool yywrap() {
 }
 
 static void checkRecursionDepth() {
-	if (contextStack.size() > maxRecursionDepth) {
-		fatalerror("Recursion limit (%zu) exceeded\n", maxRecursionDepth);
+	if (contextStack.size() > options.maxRecursionDepth) {
+		fatal("Recursion limit (%zu) exceeded", options.maxRecursionDepth);
 	}
 }
 
-static bool newFileContext(std::string const &filePath, bool updateStateNow) {
+static void newFileContext(std::string const &filePath, bool updateStateNow) {
 	checkRecursionDepth();
 
 	std::shared_ptr<std::string> uniqueIDStr = nullptr;
@@ -237,7 +236,7 @@ static bool newFileContext(std::string const &filePath, bool updateStateNow) {
 	    .macroArgs = macroArgs,
 	});
 
-	return context.lexerState.setFileAsNextState(filePath, updateStateNow);
+	context.lexerState.setFileAsNextState(filePath, updateStateNow);
 }
 
 static void newMacroContext(Symbol const &macro, std::shared_ptr<MacroArgs> macroArgs) {
@@ -253,11 +252,7 @@ static void newMacroContext(Symbol const &macro, std::shared_ptr<MacroArgs> macr
 		}
 	}
 	if (macro.src->type == NODE_REPT) {
-		std::vector<uint32_t> const &srcIters = macro.src->iters();
-		for (uint32_t i = srcIters.size(); i--;) {
-			fileInfoName.append("::REPT~");
-			fileInfoName.append(std::to_string(srcIters[i]));
-		}
+		fileInfoName.append(macro.src->reptChain());
 	}
 	fileInfoName.append("::");
 	fileInfoName.append(macro.name);
@@ -305,26 +300,34 @@ static Context &newReptContext(int32_t reptLineNo, ContentSpan const &span, uint
 	return context;
 }
 
-void fstk_RunInclude(std::string const &path, bool preInclude) {
-	std::optional<std::string> fullPath = fstk_FindFile(path);
-
-	if (!fullPath) {
-		if (generatedMissingIncludes && !preInclude) {
-			// LCOV_EXCL_START
-			if (verbose) {
-				printf("Aborting (-MG) on INCLUDE file '%s' (%s)\n", path.c_str(), strerror(errno));
-			}
-			// LCOV_EXCL_STOP
-			failedOnMissingInclude = true;
-		} else {
-			error("Unable to open included file '%s': %s\n", path.c_str(), strerror(errno));
+bool fstk_FileError(std::string const &path, char const *functionName) {
+	if (options.missingIncludeState == INC_ERROR) {
+		error("Error opening %s file '%s': %s", functionName, path.c_str(), strerror(errno));
+	} else {
+		failedOnMissingInclude = true;
+		// LCOV_EXCL_START
+		if (options.missingIncludeState == GEN_EXIT) {
+			verbosePrint(
+			    "Aborting (-MG) on %s file '%s' (%s)\n", functionName, path.c_str(), strerror(errno)
+			);
+			return true;
 		}
-		return;
+		assume(options.missingIncludeState == GEN_CONTINUE);
+		// LCOV_EXCL_STOP
 	}
+	return false;
+}
 
-	if (!newFileContext(*fullPath, false)) {
-		fatalerror("Failed to set up lexer for file include\n"); // LCOV_EXCL_LINE
+bool fstk_FailedOnMissingInclude() {
+	return failedOnMissingInclude;
+}
+
+bool fstk_RunInclude(std::string const &path) {
+	if (std::optional<std::string> fullPath = fstk_FindFile(path); fullPath) {
+		newFileContext(*fullPath, false);
+		return false;
 	}
+	return fstk_FileError(path, "INCLUDE");
 }
 
 void fstk_RunMacro(std::string const &macroName, std::shared_ptr<MacroArgs> macroArgs) {
@@ -332,14 +335,14 @@ void fstk_RunMacro(std::string const &macroName, std::shared_ptr<MacroArgs> macr
 
 	if (!macro) {
 		if (sym_IsPurgedExact(macroName)) {
-			error("Macro \"%s\" not defined; it was purged\n", macroName.c_str());
+			error("Macro \"%s\" not defined; it was purged", macroName.c_str());
 		} else {
-			error("Macro \"%s\" not defined\n", macroName.c_str());
+			error("Macro \"%s\" not defined", macroName.c_str());
 		}
 		return;
 	}
 	if (macro->type != SYM_MACRO) {
-		error("\"%s\" is not a macro\n", macroName.c_str());
+		error("\"%s\" is not a macro", macroName.c_str());
 		return;
 	}
 
@@ -372,13 +375,11 @@ void fstk_RunFor(
 	} else if (step < 0 && stop < start) {
 		count = (static_cast<int64_t>(start) - stop - 1) / -static_cast<int64_t>(step) + 1;
 	} else if (step == 0) {
-		error("FOR cannot have a step value of 0\n");
+		error("FOR cannot have a step value of 0");
 	}
 
 	if ((step > 0 && start > stop) || (step < 0 && start < stop)) {
-		warning(
-		    WARNING_BACKWARDS_FOR, "FOR goes backwards from %d to %d by %d\n", start, stop, step
-		);
+		warning(WARNING_BACKWARDS_FOR, "FOR goes backwards from %d to %d by %d", start, stop, step);
 	}
 
 	if (count == 0) {
@@ -394,7 +395,7 @@ void fstk_RunFor(
 
 bool fstk_Break() {
 	if (contextStack.top().fileInfo->type != NODE_REPT) {
-		error("BREAK can only be used inside a REPT/FOR block\n");
+		error("BREAK can only be used inside a REPT/FOR block");
 		return false;
 	}
 
@@ -404,19 +405,19 @@ bool fstk_Break() {
 
 void fstk_NewRecursionDepth(size_t newDepth) {
 	if (contextStack.size() > newDepth + 1) {
-		fatalerror("Recursion limit (%zu) exceeded\n", newDepth);
+		fatal("Recursion limit (%zu) exceeded", newDepth);
 	}
-	maxRecursionDepth = newDepth;
+	options.maxRecursionDepth = newDepth;
 }
 
-void fstk_Init(std::string const &mainPath, size_t maxDepth) {
-	if (!newFileContext(mainPath, true)) {
-		fatalerror("Failed to open main file\n");
-	}
+void fstk_Init(std::string const &mainPath) {
+	newFileContext(mainPath, true);
 
-	maxRecursionDepth = maxDepth;
-
-	if (!preIncludeName.empty()) {
-		fstk_RunInclude(preIncludeName, true);
+	for (std::string const &name : preIncludeNames) {
+		if (std::optional<std::string> fullPath = fstk_FindFile(name); fullPath) {
+			newFileContext(*fullPath, false);
+		} else {
+			error("Error reading pre-included file '%s': %s", name.c_str(), strerror(errno));
+		}
 	}
 }
