@@ -5,7 +5,6 @@
 #include <algorithm>
 #include <errno.h>
 #include <inttypes.h>
-#include <optional>
 #include <stack>
 #include <stdio.h>
 #include <stdlib.h>
@@ -22,7 +21,7 @@
 #include "asm/symbol.hpp"
 #include "asm/warning.hpp"
 
-uint8_t fillByte;
+using namespace std::literals;
 
 struct UnionStackEntry {
 	uint32_t start;
@@ -38,29 +37,30 @@ struct SectionStackEntry {
 	std::stack<UnionStackEntry> unionStack;
 };
 
-std::stack<UnionStackEntry> currentUnionStack;
-std::deque<SectionStackEntry> sectionStack;
-std::deque<Section> sectionList;
-std::unordered_map<std::string, size_t> sectionMap; // Indexes into `sectionList`
-uint32_t curOffset; // Offset into the current section (see sect_GetSymbolOffset)
-Section *currentSection = nullptr;
+static Section *currentSection = nullptr;
+static std::deque<Section> sectionList;
+static std::unordered_map<std::string, size_t> sectionMap; // Indexes into `sectionList`
+
+static uint32_t curOffset; // Offset into the current section (see `sect_GetSymbolOffset`)
+
+static std::deque<SectionStackEntry> sectionStack;
+
 static Section *currentLoadSection = nullptr;
 static std::pair<Symbol const *, Symbol const *> currentLoadLabelScopes = {nullptr, nullptr};
-int32_t loadOffset; // Offset into the LOAD section's parent (see sect_GetOutputOffset)
+static int32_t loadOffset; // Offset into the LOAD section's parent (see sect_GetOutputOffset)
+
+static std::stack<UnionStackEntry> currentUnionStack;
 
-// A quick check to see if we have an initialized section
 [[nodiscard]]
 static bool requireSection() {
 	if (currentSection) {
 		return true;
 	}
 
-	error("Cannot output data outside of a SECTION\n");
+	error("Cannot output data outside of a SECTION");
 	return false;
 }
 
-// A quick check to see if we have an initialized section that can contain
-// this much initialized data
 [[nodiscard]]
 static bool requireCodeSection() {
 	if (!requireSection()) {
@@ -72,18 +72,26 @@ static bool requireCodeSection() {
 	}
 
 	error(
-	    "Section '%s' cannot contain code or data (not ROM0 or ROMX)\n",
-	    currentSection->name.c_str()
+	    "Section '%s' cannot contain code or data (not ROM0 or ROMX)", currentSection->name.c_str()
 	);
 	return false;
 }
 
+size_t sect_CountSections() {
+	return sectionList.size();
+}
+
+void sect_ForEach(void (*callback)(Section &)) {
+	for (Section &sect : sectionList) {
+		callback(sect);
+	}
+}
+
 void sect_CheckSizes() {
 	for (Section const &sect : sectionList) {
 		if (uint32_t maxSize = sectionTypeInfo[sect.type].size; sect.size > maxSize) {
 			error(
-			    "Section '%s' grew too big (max size = 0x%" PRIX32 " bytes, reached 0x%" PRIX32
-			    ")\n",
+			    "Section '%s' grew too big (max size = 0x%" PRIX32 " bytes, reached 0x%" PRIX32 ")",
 			    sect.name.c_str(),
 			    maxSize,
 			    sect.size
@@ -101,7 +109,7 @@ Section *sect_FindSectionByName(std::string const &name) {
 #define sectError(...) \
 	do { \
 		error(__VA_ARGS__); \
-		nbSectErrors++; \
+		++nbSectErrors; \
 	} while (0)
 
 static unsigned int mergeSectUnion(
@@ -113,18 +121,18 @@ static unsigned int mergeSectUnion(
 	// Unionized sections only need "compatible" constraints, and they end up with the strictest
 	// combination of both.
 	if (sect_HasData(type)) {
-		sectError("Cannot declare ROM sections as UNION\n");
+		sectError("Cannot declare ROM sections as UNION");
 	}
 
 	if (org != UINT32_MAX) {
 		// If both are fixed, they must be the same
 		if (sect.org != UINT32_MAX && sect.org != org) {
 			sectError(
-			    "Section already declared as fixed at different address $%04" PRIx32 "\n", sect.org
+			    "Section already declared as fixed at different address $%04" PRIx32, sect.org
 			);
 		} else if (sect.align != 0 && (mask(sect.align) & (org - sect.alignOfs))) {
 			sectError(
-			    "Section already declared as aligned to %u bytes (offset %" PRIu16 ")\n",
+			    "Section already declared as aligned to %u bytes (offset %" PRIu16 ")",
 			    1U << sect.align,
 			    sect.alignOfs
 			);
@@ -138,15 +146,14 @@ static unsigned int mergeSectUnion(
 		if (sect.org != UINT32_MAX) {
 			if ((sect.org - alignOffset) & mask(alignment)) {
 				sectError(
-				    "Section already declared as fixed at incompatible address $%04" PRIx32 "\n",
+				    "Section already declared as fixed at incompatible address $%04" PRIx32,
 				    sect.org
 				);
 			}
 			// Check if alignment offsets are compatible
 		} else if ((alignOffset & mask(sect.align)) != (sect.alignOfs & mask(alignment))) {
 			sectError(
-			    "Section already declared with incompatible %u"
-			    "-byte alignment (offset %" PRIu16 ")\n",
+			    "Section already declared with incompatible %u-byte alignment (offset %" PRIu16 ")",
 			    1U << sect.align,
 			    sect.alignOfs
 			);
@@ -174,12 +181,11 @@ static unsigned int
 		// If both are fixed, they must be the same
 		if (sect.org != UINT32_MAX && sect.org != curOrg) {
 			sectError(
-			    "Section already declared as fixed at incompatible address $%04" PRIx32 "\n",
-			    sect.org
+			    "Section already declared as fixed at incompatible address $%04" PRIx32, sect.org
 			);
 		} else if (sect.align != 0 && (mask(sect.align) & (curOrg - sect.alignOfs))) {
 			sectError(
-			    "Section already declared as aligned to %u bytes (offset %" PRIu16 ")\n",
+			    "Section already declared as aligned to %u bytes (offset %" PRIu16 ")",
 			    1U << sect.align,
 			    sect.alignOfs
 			);
@@ -199,15 +205,14 @@ static unsigned int
 		if (sect.org != UINT32_MAX) {
 			if ((sect.org - curOfs) & mask(alignment)) {
 				sectError(
-				    "Section already declared as fixed at incompatible address $%04" PRIx32 "\n",
+				    "Section already declared as fixed at incompatible address $%04" PRIx32,
 				    sect.org
 				);
 			}
 			// Check if alignment offsets are compatible
 		} else if ((curOfs & mask(sect.align)) != (sect.alignOfs & mask(alignment))) {
 			sectError(
-			    "Section already declared with incompatible %u"
-			    "-byte alignment (offset %" PRIu16 ")\n",
+			    "Section already declared with incompatible %u-byte alignment (offset %" PRIu16 ")",
 			    1U << sect.align,
 			    sect.alignOfs
 			);
@@ -234,12 +239,12 @@ static void mergeSections(
 
 	if (type != sect.type) {
 		sectError(
-		    "Section already exists but with type %s\n", sectionTypeInfo[sect.type].name.c_str()
+		    "Section already exists but with type %s", sectionTypeInfo[sect.type].name.c_str()
 		);
 	}
 
 	if (sect.modifier != mod) {
-		sectError("Section already declared as SECTION %s\n", sectionModNames[sect.modifier]);
+		sectError("Section already declared as SECTION %s", sectionModNames[sect.modifier]);
 	} else {
 		switch (mod) {
 		case SECTION_UNION:
@@ -256,21 +261,22 @@ static void mergeSections(
 			}
 			// If both specify a bank, it must be the same one
 			else if (bank != UINT32_MAX && sect.bank != bank) {
-				sectError("Section already declared with different bank %" PRIu32 "\n", sect.bank);
+				sectError("Section already declared with different bank %" PRIu32, sect.bank);
 			}
 			break;
 
 		case SECTION_NORMAL:
-			sectError("Section already defined previously at ");
-			sect.src->dump(sect.fileLine);
-			putc('\n', stderr);
+			sectError([&]() {
+				fputs("Section already defined previously at ", stderr);
+				sect.src->dump(sect.fileLine);
+			});
 			break;
 		}
 	}
 
 	if (nbSectErrors) {
-		fatalerror(
-		    "Cannot create section \"%s\" (%u error%s)\n",
+		fatal(
+		    "Cannot create section \"%s\" (%u error%s)",
 		    sect.name.c_str(),
 		    nbSectErrors,
 		    nbSectErrors == 1 ? "" : "s"
@@ -280,7 +286,6 @@ static void mergeSections(
 
 #undef sectError
 
-// Create a new section, not yet in the list.
 static Section *createSection(
     std::string const &name,
     SectionType type,
@@ -315,7 +320,31 @@ static Section *createSection(
 	return &sect;
 }
 
-// Find a section by name and type. If it doesn't exist, create it.
+static Section *createSectionFragmentLiteral(Section const &parent) {
+	// Add the new section to the list, but do not update the map
+	Section &sect = sectionList.emplace_back();
+	assume(sectionMap.find(parent.name) != sectionMap.end());
+
+	sect.name = parent.name;
+	sect.type = parent.type;
+	sect.modifier = SECTION_FRAGMENT;
+	sect.src = fstk_GetFileStack();
+	sect.fileLine = lexer_GetLineNo();
+	sect.size = 0;
+	sect.org = UINT32_MAX;
+	sect.bank = parent.bank == 0 ? UINT32_MAX : parent.bank;
+	sect.align = 0;
+	sect.alignOfs = 0;
+
+	out_RegisterNode(sect.src);
+
+	// Section fragment literals must be ROM sections.
+	assume(sect_HasData(sect.type));
+	sect.data.resize(sectionTypeInfo[sect.type].size);
+
+	return &sect;
+}
+
 static Section *getSection(
     std::string const &name,
     SectionType type,
@@ -332,11 +361,11 @@ static Section *getSection(
 	if (bank != UINT32_MAX) {
 		if (type != SECTTYPE_ROMX && type != SECTTYPE_VRAM && type != SECTTYPE_SRAM
 		    && type != SECTTYPE_WRAMX) {
-			error("BANK only allowed for ROMX, WRAMX, SRAM, or VRAM sections\n");
+			error("BANK only allowed for ROMX, WRAMX, SRAM, or VRAM sections");
 		} else if (bank < sectionTypeInfo[type].firstBank
 		           || bank > sectionTypeInfo[type].lastBank) {
 			error(
-			    "%s bank value $%04" PRIx32 " out of range ($%04" PRIx32 " to $%04" PRIx32 ")\n",
+			    "%s bank value $%04" PRIx32 " out of range ($%04" PRIx32 " to $%04" PRIx32 ")",
 			    sectionTypeInfo[type].name.c_str(),
 			    bank,
 			    sectionTypeInfo[type].firstBank,
@@ -350,7 +379,7 @@ static Section *getSection(
 
 	if (alignOffset >= 1 << alignment) {
 		error(
-		    "Alignment offset (%" PRIu16 ") must be smaller than alignment size (%u)\n",
+		    "Alignment offset (%" PRIu16 ") must be smaller than alignment size (%u)",
 		    alignOffset,
 		    1U << alignment
 		);
@@ -361,7 +390,7 @@ static Section *getSection(
 		if (org < sectionTypeInfo[type].startAddr || org > endaddr(type)) {
 			error(
 			    "Section \"%s\"'s fixed address $%04" PRIx32 " is outside of range [$%04" PRIx16
-			    "; $%04" PRIx16 "]\n",
+			    "; $%04" PRIx16 "]",
 			    name.c_str(),
 			    org,
 			    sectionTypeInfo[type].startAddr,
@@ -372,7 +401,7 @@ static Section *getSection(
 
 	if (alignment != 0) {
 		if (alignment > 16) {
-			error("Alignment must be between 0 and 16, not %u\n", alignment);
+			error("Alignment must be between 0 and 16, not %u", alignment);
 			alignment = 16;
 		}
 		// It doesn't make sense to have both alignment and org set
@@ -380,12 +409,12 @@ static Section *getSection(
 
 		if (org != UINT32_MAX) {
 			if ((org - alignOffset) & mask) {
-				error("Section \"%s\"'s fixed address doesn't match its alignment\n", name.c_str());
+				error("Section \"%s\"'s fixed address doesn't match its alignment", name.c_str());
 			}
 			alignment = 0; // Ignore it if it's satisfied
 		} else if (sectionTypeInfo[type].startAddr & mask) {
 			error(
-			    "Section \"%s\"'s alignment cannot be attained in %s\n",
+			    "Section \"%s\"'s alignment cannot be attained in %s",
 			    name.c_str(),
 			    sectionTypeInfo[type].name.c_str()
 			);
@@ -412,15 +441,24 @@ static Section *getSection(
 	return sect;
 }
 
-// Set the current section
 static void changeSection() {
 	if (!currentUnionStack.empty()) {
-		fatalerror("Cannot change the section within a UNION\n");
+		fatal("Cannot change the section within a UNION");
 	}
 
 	sym_ResetCurrentLabelScopes();
 }
 
+uint32_t Section::getID() const {
+	// Section fragments share the same name but have different IDs, so search by identity
+	if (auto search =
+	        std::find_if(RANGE(sectionList), [this](Section const &s) { return &s == this; });
+	    search != sectionList.end()) {
+		return static_cast<uint32_t>(std::distance(sectionList.begin(), search));
+	}
+	return UINT32_MAX; // LCOV_EXCL_LINE
+}
+
 bool Section::isSizeKnown() const {
 	// SECTION UNION and SECTION FRAGMENT can still grow
 	if (modifier != SECTION_NORMAL) {
@@ -442,7 +480,6 @@ bool Section::isSizeKnown() const {
 	return true;
 }
 
-// Set the current section by name and type
 void sect_NewSection(
     std::string const &name,
     SectionType type,
@@ -452,7 +489,7 @@ void sect_NewSection(
 ) {
 	for (SectionStackEntry &entry : sectionStack) {
 		if (entry.section && entry.section->name == name) {
-			fatalerror("Section '%s' is already on the stack\n", name.c_str());
+			fatal("Section '%s' is already on the stack", name.c_str());
 		}
 	}
 
@@ -468,7 +505,6 @@ void sect_NewSection(
 	currentSection = sect;
 }
 
-// Set the current section by name and type
 void sect_SetLoadSection(
     std::string const &name,
     SectionType type,
@@ -486,7 +522,7 @@ void sect_SetLoadSection(
 	}
 
 	if (sect_HasData(type)) {
-		error("`LOAD` blocks cannot create a ROM section\n");
+		error("`LOAD` blocks cannot create a ROM section");
 		return;
 	}
 
@@ -505,13 +541,11 @@ void sect_SetLoadSection(
 
 void sect_EndLoadSection(char const *cause) {
 	if (cause) {
-		warning(
-		    WARNING_UNTERMINATED_LOAD, "`LOAD` block without `ENDL` terminated by `%s`\n", cause
-		);
+		warning(WARNING_UNTERMINATED_LOAD, "`LOAD` block without `ENDL` terminated by `%s`", cause);
 	}
 
 	if (!currentLoadSection) {
-		error("Found `ENDL` outside of a `LOAD` block\n");
+		error("Found `ENDL` outside of a `LOAD` block");
 		return;
 	}
 
@@ -524,7 +558,7 @@ void sect_EndLoadSection(char const *cause) {
 
 void sect_CheckLoadClosed() {
 	if (currentLoadSection) {
-		warning(WARNING_UNTERMINATED_LOAD, "`LOAD` block without `ENDL` terminated by EOF\n");
+		warning(WARNING_UNTERMINATED_LOAD, "`LOAD` block without `ENDL` terminated by EOF");
 	}
 }
 
@@ -532,7 +566,6 @@ Section *sect_GetSymbolSection() {
 	return currentLoadSection ? currentLoadSection : currentSection;
 }
 
-// The offset into the section above
 uint32_t sect_GetSymbolOffset() {
 	return curOffset;
 }
@@ -541,6 +574,14 @@ uint32_t sect_GetOutputOffset() {
 	return curOffset + loadOffset;
 }
 
+std::optional<uint32_t> sect_GetOutputBank() {
+	return currentSection ? std::optional<uint32_t>(currentSection->bank) : std::nullopt;
+}
+
+Patch *sect_AddOutputPatch() {
+	return currentSection ? &currentSection->patches.emplace_front() : nullptr;
+}
+
 // Returns how many bytes need outputting for the specified alignment and offset to succeed
 uint32_t sect_GetAlignBytes(uint8_t alignment, uint16_t offset) {
 	Section *sect = sect_GetSymbolSection();
@@ -572,37 +613,50 @@ void sect_AlignPC(uint8_t alignment, uint16_t offset) {
 	uint32_t alignSize = 1 << alignment; // Size of an aligned "block"
 
 	if (sect->org != UINT32_MAX) {
-		if ((sect->org + curOffset - offset) % alignSize) {
+		if (uint32_t actualOffset = (sect->org + curOffset) % alignSize; actualOffset != offset) {
 			error(
-			    "Section's fixed address fails required alignment (PC = $%04" PRIx32 ")\n",
-			    sect->org + curOffset
+			    "Section is misaligned (at PC = $%04" PRIx32 ", expected ALIGN[%" PRIu32
+			    ", %" PRIu32 "], got ALIGN[%" PRIu32 ", %" PRIu32 "])",
+			    sect->org + curOffset,
+			    alignment,
+			    offset,
+			    alignment,
+			    actualOffset
 			);
 		}
-	} else if (sect->align != 0
-	           && (((sect->alignOfs + curOffset) % (1u << sect->align)) - offset) % alignSize) {
-		error(
-		    "Section's alignment fails required alignment (offset from section start = $%04" PRIx32
-		    ")\n",
-		    curOffset
-		);
-	} else if (alignment >= 16) {
-		// Treat an alignment large enough as fixing the address.
-		// Note that this also ensures that a section's alignment never becomes 16 or greater.
-		if (alignment > 16) {
-			error("Alignment must be between 0 and 16, not %u\n", alignment);
+	} else {
+		if (uint32_t actualOffset = (sect->alignOfs + curOffset) % alignSize,
+		    sectAlignSize = 1 << sect->align;
+		    sect->align != 0 && actualOffset % sectAlignSize != offset % sectAlignSize) {
+			error(
+			    "Section is misaligned ($%04" PRIx32
+			    " bytes into the section, expected ALIGN[%" PRIu32 ", %" PRIu32
+			    "], got ALIGN[%" PRIu32 ", %" PRIu32 "])",
+			    curOffset,
+			    alignment,
+			    offset,
+			    alignment,
+			    actualOffset
+			);
+		} else if (alignment >= 16) {
+			// Treat an alignment large enough as fixing the address.
+			// Note that this also ensures that a section's alignment never becomes 16 or greater.
+			if (alignment > 16) {
+				error("Alignment must be between 0 and 16, not %u", alignment);
+			}
+			sect->align = 0; // Reset the alignment, since we're fixing the address.
+			sect->org = offset - curOffset;
+		} else if (alignment > sect->align) {
+			sect->align = alignment;
+			// We need `(sect->alignOfs + curOffset) % alignSize == offset`
+			sect->alignOfs = (offset - curOffset) % alignSize;
 		}
-		sect->align = 0; // Reset the alignment, since we're fixing the address.
-		sect->org = offset - curOffset;
-	} else if (alignment > sect->align) {
-		sect->align = alignment;
-		// We need `(sect->alignOfs + curOffset) % alignSize == offset`
-		sect->alignOfs = (offset - curOffset) % alignSize;
 	}
 }
 
 static void growSection(uint32_t growth) {
 	if (growth > 0 && curOffset > UINT32_MAX - growth) {
-		fatalerror("Section size would overflow internal counter\n");
+		fatal("Section size would overflow internal counter");
 	}
 	curOffset += growth;
 	if (uint32_t outOffset = sect_GetOutputOffset(); outOffset > currentSection->size) {
@@ -643,11 +697,11 @@ void sect_StartUnion() {
 	// your own peril! ^^
 
 	if (!currentSection) {
-		error("UNIONs must be inside a SECTION\n");
+		error("UNIONs must be inside a SECTION");
 		return;
 	}
 	if (sect_HasData(currentSection->type)) {
-		error("Cannot use UNION inside of ROM0 or ROMX sections\n");
+		error("Cannot use UNION inside of ROM0 or ROMX sections");
 		return;
 	}
 
@@ -666,7 +720,7 @@ static void endUnionMember() {
 
 void sect_NextUnionMember() {
 	if (currentUnionStack.empty()) {
-		error("Found NEXTU outside of a UNION construct\n");
+		error("Found NEXTU outside of a UNION construct");
 		return;
 	}
 	endUnionMember();
@@ -674,7 +728,7 @@ void sect_NextUnionMember() {
 
 void sect_EndUnion() {
 	if (currentUnionStack.empty()) {
-		error("Found ENDU outside of a UNION construct\n");
+		error("Found ENDU outside of a UNION construct");
 		return;
 	}
 	endUnionMember();
@@ -684,11 +738,10 @@ void sect_EndUnion() {
 
 void sect_CheckUnionClosed() {
 	if (!currentUnionStack.empty()) {
-		error("Unterminated UNION construct\n");
+		error("Unterminated UNION construct");
 	}
 }
 
-// Output a constant byte
 void sect_ConstByte(uint8_t byte) {
 	if (!requireCodeSection()) {
 		return;
@@ -697,52 +750,48 @@ void sect_ConstByte(uint8_t byte) {
 	writeByte(byte);
 }
 
-// Output a string's character units as bytes
-void sect_ByteString(std::vector<int32_t> const &string) {
+void sect_ByteString(std::vector<int32_t> const &str) {
 	if (!requireCodeSection()) {
 		return;
 	}
 
-	for (int32_t unit : string) {
+	for (int32_t unit : str) {
 		if (!checkNBit(unit, 8, "All character units")) {
 			break;
 		}
 	}
 
-	for (int32_t unit : string) {
+	for (int32_t unit : str) {
 		writeByte(static_cast<uint8_t>(unit));
 	}
 }
 
-// Output a string's character units as words
-void sect_WordString(std::vector<int32_t> const &string) {
+void sect_WordString(std::vector<int32_t> const &str) {
 	if (!requireCodeSection()) {
 		return;
 	}
 
-	for (int32_t unit : string) {
+	for (int32_t unit : str) {
 		if (!checkNBit(unit, 16, "All character units")) {
 			break;
 		}
 	}
 
-	for (int32_t unit : string) {
+	for (int32_t unit : str) {
 		writeWord(static_cast<uint16_t>(unit));
 	}
 }
 
-// Output a string's character units as longs
-void sect_LongString(std::vector<int32_t> const &string) {
+void sect_LongString(std::vector<int32_t> const &str) {
 	if (!requireCodeSection()) {
 		return;
 	}
 
-	for (int32_t unit : string) {
+	for (int32_t unit : str) {
 		writeLong(static_cast<uint32_t>(unit));
 	}
 }
 
-// Skip this many bytes
 void sect_Skip(uint32_t skip, bool ds) {
 	if (!requireSection()) {
 		return;
@@ -754,7 +803,7 @@ void sect_Skip(uint32_t skip, bool ds) {
 		if (!ds) {
 			warning(
 			    WARNING_EMPTY_DATA_DIRECTIVE,
-			    "%s directive without data in ROM\n",
+			    "%s directive without data in ROM",
 			    (skip == 4)   ? "DL"
 			    : (skip == 2) ? "DW"
 			                  : "DB"
@@ -762,12 +811,11 @@ void sect_Skip(uint32_t skip, bool ds) {
 		}
 		// We know we're in a code SECTION
 		while (skip--) {
-			writeByte(fillByte);
+			writeByte(options.padByte);
 		}
 	}
 }
 
-// Output a byte that can be relocatable or constant
 void sect_RelByte(Expression const &expr, uint32_t pcShift) {
 	if (!requireCodeSection()) {
 		return;
@@ -781,13 +829,12 @@ void sect_RelByte(Expression const &expr, uint32_t pcShift) {
 	}
 }
 
-// Output several bytes that can be relocatable or constant
 void sect_RelBytes(uint32_t n, std::vector<Expression> const &exprs) {
 	if (!requireCodeSection()) {
 		return;
 	}
 
-	for (uint32_t i = 0; i < n; i++) {
+	for (uint32_t i = 0; i < n; ++i) {
 		if (Expression const &expr = exprs[i % exprs.size()]; !expr.isKnown()) {
 			createPatch(PATCHTYPE_BYTE, expr, i);
 			writeByte(0);
@@ -797,7 +844,6 @@ void sect_RelBytes(uint32_t n, std::vector<Expression> const &exprs) {
 	}
 }
 
-// Output a word that can be relocatable or constant
 void sect_RelWord(Expression const &expr, uint32_t pcShift) {
 	if (!requireCodeSection()) {
 		return;
@@ -811,7 +857,6 @@ void sect_RelWord(Expression const &expr, uint32_t pcShift) {
 	}
 }
 
-// Output a long that can be relocatable or constant
 void sect_RelLong(Expression const &expr, uint32_t pcShift) {
 	if (!requireCodeSection()) {
 		return;
@@ -825,7 +870,6 @@ void sect_RelLong(Expression const &expr, uint32_t pcShift) {
 	}
 }
 
-// Output a PC-relative byte that can be relocatable or constant
 void sect_PCRelByte(Expression const &expr, uint32_t pcShift) {
 	if (!requireCodeSection()) {
 		return;
@@ -849,7 +893,7 @@ void sect_PCRelByte(Expression const &expr, uint32_t pcShift) {
 		if (offset < -128 || offset > 127) {
 			error(
 			    "JR target must be between -128 and 127 bytes away, not %" PRId16
-			    "; use JP instead\n",
+			    "; use JP instead",
 			    offset
 			);
 			writeByte(0);
@@ -859,14 +903,9 @@ void sect_PCRelByte(Expression const &expr, uint32_t pcShift) {
 	}
 }
 
-// Output a binary file
-void sect_BinaryFile(std::string const &name, int32_t startPos) {
-	if (startPos < 0) {
-		error("Start position cannot be negative (%" PRId32 ")\n", startPos);
-		startPos = 0;
-	}
+bool sect_BinaryFile(std::string const &name, uint32_t startPos) {
 	if (!requireCodeSection()) {
-		return;
+		return false;
 	}
 
 	FILE *file = nullptr;
@@ -874,40 +913,26 @@ void sect_BinaryFile(std::string const &name, int32_t startPos) {
 		file = fopen(fullPath->c_str(), "rb");
 	}
 	if (!file) {
-		if (generatedMissingIncludes) {
-			// LCOV_EXCL_START
-			if (verbose) {
-				printf("Aborting (-MG) on INCBIN file '%s' (%s)\n", name.c_str(), strerror(errno));
-			}
-			// LCOV_EXCL_STOP
-			failedOnMissingInclude = true;
-		} else {
-			error("Error opening INCBIN file '%s': %s\n", name.c_str(), strerror(errno));
-		}
-		return;
+		return fstk_FileError(name, "INCBIN");
 	}
 	Defer closeFile{[&] { fclose(file); }};
 
-	if (fseek(file, 0, SEEK_END) != -1) {
+	if (fseek(file, 0, SEEK_END) == 0) {
 		if (startPos > ftell(file)) {
-			error("Specified start position is greater than length of file '%s'\n", name.c_str());
-			return;
+			error("Specified start position is greater than length of file '%s'", name.c_str());
+			return false;
 		}
 		// The file is seekable; skip to the specified start position
 		fseek(file, startPos, SEEK_SET);
 	} else {
 		if (errno != ESPIPE) {
-			error(
-			    "Error determining size of INCBIN file '%s': %s\n", name.c_str(), strerror(errno)
-			);
+			error("Error determining size of INCBIN file '%s': %s", name.c_str(), strerror(errno));
 		}
 		// The file isn't seekable, so we'll just skip bytes one at a time
 		while (startPos--) {
 			if (fgetc(file) == EOF) {
-				error(
-				    "Specified start position is greater than length of file '%s'\n", name.c_str()
-				);
-				return;
+				error("Specified start position is greater than length of file '%s'", name.c_str());
+				return false;
 			}
 		}
 	}
@@ -917,25 +942,17 @@ void sect_BinaryFile(std::string const &name, int32_t startPos) {
 	}
 
 	if (ferror(file)) {
-		error("Error reading INCBIN file '%s': %s\n", name.c_str(), strerror(errno));
+		error("Error reading INCBIN file '%s': %s", name.c_str(), strerror(errno));
 	}
+	return false;
 }
 
-// Output a slice of a binary file
-void sect_BinaryFileSlice(std::string const &name, int32_t startPos, int32_t length) {
-	if (startPos < 0) {
-		error("Start position cannot be negative (%" PRId32 ")\n", startPos);
-		startPos = 0;
-	}
-	if (length < 0) {
-		error("Number of bytes to read cannot be negative (%" PRId32 ")\n", length);
-		length = 0;
-	}
+bool sect_BinaryFileSlice(std::string const &name, uint32_t startPos, uint32_t length) {
 	if (!requireCodeSection()) {
-		return;
+		return false;
 	}
 	if (length == 0) { // Don't even bother with 0-byte slices
-		return;
+		return false;
 	}
 
 	FILE *file = nullptr;
@@ -943,50 +960,36 @@ void sect_BinaryFileSlice(std::string const &name, int32_t startPos, int32_t len
 		file = fopen(fullPath->c_str(), "rb");
 	}
 	if (!file) {
-		if (generatedMissingIncludes) {
-			// LCOV_EXCL_START
-			if (verbose) {
-				printf("Aborting (-MG) on INCBIN file '%s' (%s)\n", name.c_str(), strerror(errno));
-			}
-			// LCOV_EXCL_STOP
-			failedOnMissingInclude = true;
-		} else {
-			error("Error opening INCBIN file '%s': %s\n", name.c_str(), strerror(errno));
-		}
-		return;
+		return fstk_FileError(name, "INCBIN");
 	}
 	Defer closeFile{[&] { fclose(file); }};
 
-	if (fseek(file, 0, SEEK_END) != -1) {
-		if (int32_t fsize = ftell(file); startPos > fsize) {
-			error("Specified start position is greater than length of file '%s'\n", name.c_str());
-			return;
+	if (fseek(file, 0, SEEK_END) == 0) {
+		if (long fsize = ftell(file); startPos > fsize) {
+			error("Specified start position is greater than length of file '%s'", name.c_str());
+			return false;
 		} else if (startPos + length > fsize) {
 			error(
 			    "Specified range in INCBIN file '%s' is out of bounds (%" PRIu32 " + %" PRIu32
-			    " > %" PRIu32 ")\n",
+			    " > %ld)",
 			    name.c_str(),
 			    startPos,
 			    length,
 			    fsize
 			);
-			return;
+			return false;
 		}
 		// The file is seekable; skip to the specified start position
 		fseek(file, startPos, SEEK_SET);
 	} else {
 		if (errno != ESPIPE) {
-			error(
-			    "Error determining size of INCBIN file '%s': %s\n", name.c_str(), strerror(errno)
-			);
+			error("Error determining size of INCBIN file '%s': %s", name.c_str(), strerror(errno));
 		}
 		// The file isn't seekable, so we'll just skip bytes one at a time
 		while (startPos--) {
 			if (fgetc(file) == EOF) {
-				error(
-				    "Specified start position is greater than length of file '%s'\n", name.c_str()
-				);
-				return;
+				error("Specified start position is greater than length of file '%s'", name.c_str());
+				return false;
 			}
 		}
 	}
@@ -995,18 +998,18 @@ void sect_BinaryFileSlice(std::string const &name, int32_t startPos, int32_t len
 		if (int byte = fgetc(file); byte != EOF) {
 			writeByte(byte);
 		} else if (ferror(file)) {
-			error("Error reading INCBIN file '%s': %s\n", name.c_str(), strerror(errno));
+			error("Error reading INCBIN file '%s': %s", name.c_str(), strerror(errno));
 		} else {
 			error(
-			    "Premature end of INCBIN file '%s' (%" PRId32 " bytes left to read)\n",
+			    "Premature end of INCBIN file '%s' (%" PRId32 " bytes left to read)",
 			    name.c_str(),
 			    length + 1
 			);
 		}
 	}
+	return false;
 }
 
-// Section stack routines
 void sect_PushSection() {
 	sectionStack.push_front({
 	    .section = currentSection,
@@ -1026,7 +1029,7 @@ void sect_PushSection() {
 
 void sect_PopSection() {
 	if (sectionStack.empty()) {
-		fatalerror("No entries in the section stack\n");
+		fatal("No entries in the section stack");
 	}
 
 	if (currentLoadSection) {
@@ -1047,17 +1050,17 @@ void sect_PopSection() {
 
 void sect_CheckStack() {
 	if (!sectionStack.empty()) {
-		warning(WARNING_UNMATCHED_DIRECTIVE, "`PUSHS` without corresponding `POPS`\n");
+		warning(WARNING_UNMATCHED_DIRECTIVE, "`PUSHS` without corresponding `POPS`");
 	}
 }
 
 void sect_EndSection() {
 	if (!currentSection) {
-		fatalerror("Cannot end the section outside of a SECTION\n");
+		fatal("Cannot end the section outside of a SECTION");
 	}
 
 	if (!currentUnionStack.empty()) {
-		fatalerror("Cannot end the section within a UNION\n");
+		fatal("Cannot end the section within a UNION");
 	}
 
 	if (currentLoadSection) {
@@ -1068,3 +1071,40 @@ void sect_EndSection() {
 	currentSection = nullptr;
 	sym_ResetCurrentLabelScopes();
 }
+
+std::string sect_PushSectionFragmentLiteral() {
+	static uint64_t nextFragmentLiteralID = 0;
+
+	// Like `requireCodeSection` but fatal
+	if (!currentSection) {
+		fatal("Cannot output fragment literals outside of a SECTION");
+	}
+	if (!sect_HasData(currentSection->type)) {
+		fatal(
+		    "Section '%s' cannot contain fragment literals (not ROM0 or ROMX)",
+		    currentSection->name.c_str()
+		);
+	}
+
+	if (currentLoadSection) {
+		fatal("`LOAD` blocks cannot contain fragment literals");
+	}
+	if (currentSection->modifier == SECTION_UNION) {
+		fatal("`SECTION UNION` cannot contain fragment literals");
+	}
+
+	// A section containing a fragment literal has to become a fragment too
+	currentSection->modifier = SECTION_FRAGMENT;
+
+	Section *parent = currentSection;
+	sect_PushSection(); // Resets `currentSection`
+
+	Section *sect = createSectionFragmentLiteral(*parent);
+
+	changeSection();
+	curOffset = sect->size;
+	currentSection = sect;
+
+	// Return a symbol ID to use for the address of this section fragment
+	return "$"s + std::to_string(nextFragmentLiteralID++);
+}
