@@ -3,12 +3,13 @@
 #include "asm/symbol.hpp"
 
 #include <algorithm>
+#include <errno.h>
 #include <inttypes.h>
 #include <stdio.h>
 #include <unordered_map>
 #include <unordered_set>
 
-#include "error.hpp"
+#include "diagnostics.hpp"
 #include "helpers.hpp" // assume
 #include "util.hpp"
 #include "version.hpp"
@@ -21,21 +22,24 @@
 
 using namespace std::literals;
 
-std::unordered_map<std::string, Symbol> symbols;
-std::unordered_set<std::string> purgedSymbols;
+static std::unordered_map<std::string, Symbol> symbols;
+static std::unordered_set<std::string> purgedSymbols;
 
 static Symbol const *globalScope = nullptr; // Current section's global label scope
 static Symbol const *localScope = nullptr;  // Current section's local label scope
+
 static Symbol *PCSymbol;
 static Symbol *NARGSymbol;
 static Symbol *globalScopeSymbol;
 static Symbol *localScopeSymbol;
 static Symbol *RSSymbol;
+
 static char savedTIME[256];
 static char savedDATE[256];
 static char savedTIMESTAMP_ISO8601_LOCAL[256];
 static char savedTIMESTAMP_ISO8601_UTC[256];
-static bool exportAll;
+
+static bool exportAll = false; // -E
 
 bool sym_IsPC(Symbol const *sym) {
 	return sym == PCSymbol;
@@ -51,14 +55,14 @@ static int32_t NARGCallback() {
 	if (MacroArgs const *macroArgs = fstk_GetCurrentMacroArgs(); macroArgs) {
 		return macroArgs->nbArgs();
 	} else {
-		error("_NARG has no value outside of a macro\n");
+		error("_NARG has no value outside of a macro");
 		return 0;
 	}
 }
 
 static std::shared_ptr<std::string> globalScopeCallback() {
 	if (!globalScope) {
-		error("\".\" has no value outside of a label scope\n");
+		error("\".\" has no value outside of a label scope");
 		return std::make_shared<std::string>("");
 	}
 	return std::make_shared<std::string>(globalScope->name);
@@ -66,7 +70,7 @@ static std::shared_ptr<std::string> globalScopeCallback() {
 
 static std::shared_ptr<std::string> localScopeCallback() {
 	if (!localScope) {
-		error("\"..\" has no value outside of a local label scope\n");
+		error("\"..\" has no value outside of a local label scope");
 		return std::make_shared<std::string>("");
 	}
 	return std::make_shared<std::string>(localScope->name);
@@ -111,13 +115,13 @@ std::shared_ptr<std::string> Symbol::getEqus() const {
 }
 
 static void dumpFilename(Symbol const &sym) {
+	fputs(" at ", stderr);
 	if (sym.src) {
 		sym.src->dump(sym.fileLine);
-		putc('\n', stderr);
 	} else if (sym.isBuiltin) {
-		fputs("<builtin>\n", stderr);
+		fputs("<builtin>", stderr);
 	} else {
-		fputs("<command-line>\n", stderr);
+		fputs("<command-line>", stderr);
 	}
 }
 
@@ -140,23 +144,25 @@ static bool isValidIdentifier(std::string const &s) {
 static void alreadyDefinedError(Symbol const &sym, char const *asType) {
 	if (sym.isBuiltin && !sym_FindScopedValidSymbol(sym.name)) {
 		// `DEF()` would return false, so we should not claim the symbol is already defined
-		error("'%s' is reserved for a built-in symbol\n", sym.name.c_str());
+		error("'%s' is reserved for a built-in symbol", sym.name.c_str());
 	} else {
-		error("'%s' already defined", sym.name.c_str());
-		if (asType) {
-			fprintf(stderr, " as %s", asType);
-		}
-		fputs(" at ", stderr);
-		dumpFilename(sym);
-		if (sym.type == SYM_EQUS) {
+		error([&]() {
+			fprintf(stderr, "'%s' already defined", sym.name.c_str());
+			if (asType) {
+				fprintf(stderr, " as %s", asType);
+			}
+			dumpFilename(sym);
+			if (sym.type != SYM_EQUS) {
+				return;
+			}
 			if (std::string const &contents = *sym.getEqus(); isValidIdentifier(contents)) {
 				fprintf(
 				    stderr,
-				    "    (should it be {interpolated} to define its contents \"%s\"?)\n",
+				    "\n    (should it be {interpolated} to define its contents \"%s\"?)",
 				    contents.c_str()
 				);
 			}
-		}
+		});
 	}
 }
 
@@ -164,9 +170,9 @@ static void redefinedError(Symbol const &sym) {
 	assume(sym.isBuiltin);
 	if (!sym_FindScopedValidSymbol(sym.name)) {
 		// `DEF()` would return false, so we should not imply the symbol is already defined
-		error("'%s' is reserved for a built-in symbol\n", sym.name.c_str());
+		error("'%s' is reserved for a built-in symbol", sym.name.c_str());
 	} else {
-		error("Built-in symbol '%s' cannot be redefined\n", sym.name.c_str());
+		error("Built-in symbol '%s' cannot be redefined", sym.name.c_str());
 	}
 }
 
@@ -215,12 +221,12 @@ static bool isAutoScoped(std::string const &symName) {
 
 	// Check for nothing after the dot
 	if (dotPos == symName.length() - 1) {
-		fatalerror("'%s' is a nonsensical reference to an empty local label\n", symName.c_str());
+		fatal("'%s' is a nonsensical reference to an empty local label", symName.c_str());
 	}
 
 	// Check for more than one dot
 	if (symName.find('.', dotPos + 1) != std::string::npos) {
-		fatalerror("'%s' is a nonsensical reference to a nested local label\n", symName.c_str());
+		fatal("'%s' is a nonsensical reference to a nested local label", symName.c_str());
 	}
 
 	// Check for already-qualified local label
@@ -230,7 +236,7 @@ static bool isAutoScoped(std::string const &symName) {
 
 	// Check for unqualifiable local label
 	if (!globalScope) {
-		fatalerror("Unqualified local label '%s' in main scope\n", symName.c_str());
+		fatal("Unqualified local label '%s' in main scope", symName.c_str());
 	}
 
 	return true;
@@ -279,19 +285,19 @@ void sym_Purge(std::string const &symName) {
 
 	if (!sym) {
 		if (sym_IsPurgedScoped(symName)) {
-			error("'%s' was already purged\n", symName.c_str());
+			error("'%s' was already purged", symName.c_str());
 		} else {
-			error("'%s' not defined\n", symName.c_str());
+			error("'%s' not defined", symName.c_str());
 		}
 	} else if (sym->isBuiltin) {
-		error("Built-in symbol '%s' cannot be purged\n", symName.c_str());
+		error("Built-in symbol '%s' cannot be purged", symName.c_str());
 	} else if (sym->ID != UINT32_MAX) {
-		error("Symbol \"%s\" is referenced and thus cannot be purged\n", symName.c_str());
+		error("Symbol \"%s\" is referenced and thus cannot be purged", symName.c_str());
 	} else {
 		if (sym->isExported) {
-			warning(WARNING_PURGE_1, "Purging an exported symbol \"%s\"\n", symName.c_str());
+			warning(WARNING_PURGE_1, "Purging an exported symbol \"%s\"", symName.c_str());
 		} else if (sym->isLabel()) {
-			warning(WARNING_PURGE_2, "Purging a label \"%s\"\n", symName.c_str());
+			warning(WARNING_PURGE_2, "Purging a label \"%s\"", symName.c_str());
 		}
 		// Do not keep a reference to the label after purging it
 		if (sym == globalScope) {
@@ -331,12 +337,12 @@ uint32_t Symbol::getConstantValue() const {
 
 	if (sym_IsPC(this)) {
 		if (!getSection()) {
-			error("PC has no value outside of a section\n");
+			error("PC has no value outside of a section");
 		} else {
-			error("PC does not have a constant value; the current section is not fixed\n");
+			error("PC does not have a constant value; the current section is not fixed");
 		}
 	} else {
-		error("\"%s\" does not have a constant value\n", name.c_str());
+		error("\"%s\" does not have a constant value", name.c_str());
 	}
 	return 0;
 }
@@ -371,8 +377,10 @@ static Symbol *createNonrelocSymbol(std::string const &symName, bool numeric) {
 		return nullptr; // Don't allow overriding the symbol, that'd be bad!
 	} else if (!numeric) {
 		// The symbol has already been referenced, but it's not allowed
-		error("'%s' already referenced at ", symName.c_str());
-		dumpFilename(*sym);
+		error([&]() {
+			fprintf(stderr, "'%s' already referenced", symName.c_str());
+			dumpFilename(*sym);
+		});
 		return nullptr; // Don't allow overriding the symbol, that'd be bad!
 	}
 
@@ -437,8 +445,10 @@ Symbol *sym_RedefString(std::string const &symName, std::shared_ptr<std::string>
 		if (sym->isDefined()) {
 			alreadyDefinedError(*sym, "non-EQUS");
 		} else {
-			error("'%s' already referenced at ", symName.c_str());
-			dumpFilename(*sym);
+			error([&]() {
+				fprintf(stderr, "'%s' already referenced", symName.c_str());
+				dumpFilename(*sym);
+			});
 		}
 		return nullptr;
 	} else if (sym->isBuiltin) {
@@ -493,7 +503,7 @@ static Symbol *addLabel(std::string const &symName) {
 	sym->section = sect_GetSymbolSection();
 
 	if (sym && !sym->section) {
-		error("Label \"%s\" created outside of a SECTION\n", symName.c_str());
+		error("Label \"%s\" created outside of a SECTION", symName.c_str());
 	}
 
 	return sym;
@@ -538,7 +548,7 @@ Symbol *sym_AddAnonLabel() {
 	}
 
 	std::string anon = sym_MakeAnonLabelName(0, true); // The direction is important!
-	anonLabelID++;
+	++anonLabelID;
 	return addLabel(anon);
 }
 
@@ -549,7 +559,7 @@ std::string sym_MakeAnonLabelName(uint32_t ofs, bool neg) {
 		if (ofs > anonLabelID) {
 			error(
 			    "Reference to anonymous label %" PRIu32 " before, when only %" PRIu32
-			    " ha%s been created so far\n",
+			    " ha%s been created so far",
 			    ofs,
 			    anonLabelID,
 			    anonLabelID == 1 ? "s" : "ve"
@@ -558,12 +568,12 @@ std::string sym_MakeAnonLabelName(uint32_t ofs, bool neg) {
 			id = anonLabelID - ofs;
 		}
 	} else {
-		ofs--; // We're referencing symbols that haven't been created yet...
-		if (ofs > UINT32_MAX - anonLabelID) {
+		// We're referencing symbols that haven't been created yet...
+		if (--ofs > UINT32_MAX - anonLabelID) {
 			// LCOV_EXCL_START
 			error(
 			    "Reference to anonymous label %" PRIu32 " after, when only %" PRIu32
-			    " may still be created\n",
+			    " may still be created",
 			    ofs + 1,
 			    UINT32_MAX - anonLabelID
 			);
@@ -580,7 +590,7 @@ void sym_Export(std::string const &symName) {
 	if (symName.starts_with('!')) {
 		// LCOV_EXCL_START
 		// The parser does not accept anonymous labels for an `EXPORT` directive
-		error("Anonymous labels cannot be exported\n");
+		error("Anonymous labels cannot be exported");
 		return;
 		// LCOV_EXCL_STOP
 	}
@@ -666,7 +676,7 @@ void sym_Init(time_t now) {
 
 	// LCOV_EXCL_START
 	if (now == static_cast<time_t>(-1)) {
-		warn("Failed to determine current time");
+		warnx("Failed to determine current time: %s", strerror(errno));
 		// Fall back by pretending we are at the Epoch
 		now = 0;
 	}
