@@ -1,6 +1,6 @@
 // SPDX-License-Identifier: MIT
 
-// This implementation was taken from musl and modified for RGBDS
+// This implementation was taken from musl and modified for RGBDS.
 
 #include "extern/getopt.hpp"
 
@@ -57,7 +57,7 @@ static int getopt(int argc, char *argv[], char const *optstring) {
 	}
 
 	if (!musl_optpos) {
-		musl_optpos++;
+		++musl_optpos;
 	}
 	k = mbtowc(&c, argv[musl_optind] + musl_optpos, MB_LEN_MAX);
 	if (k < 0) {
@@ -68,12 +68,12 @@ static int getopt(int argc, char *argv[], char const *optstring) {
 	musl_optpos += k;
 
 	if (!argv[musl_optind][musl_optpos]) {
-		musl_optind++;
+		++musl_optind;
 		musl_optpos = 0;
 	}
 
 	if (optstring[0] == '-' || optstring[0] == '+') {
-		optstring++;
+		++optstring;
 	}
 
 	i = 0;
@@ -83,7 +83,7 @@ static int getopt(int argc, char *argv[], char const *optstring) {
 		if (l > 0) {
 			i += l;
 		} else {
-			i++;
+			++i;
 		}
 	} while (l && d != c);
 
@@ -118,7 +118,7 @@ static void permute(char **argv, int dest, int src) {
 	char *tmp = argv[src];
 	int i;
 
-	for (i = src; i > dest; i--) {
+	for (i = src; i > dest; --i) {
 		argv[i] = argv[i - 1];
 	}
 	argv[dest] = tmp;
@@ -146,7 +146,7 @@ static int musl_getopt_long(
 	skipped = musl_optind;
 	if (optstring[0] != '+' && optstring[0] != '-') {
 		int i;
-		for (i = musl_optind;; i++) {
+		for (i = musl_optind;; ++i) {
 			if (i >= argc || !argv[i]) {
 				return -1;
 			}
@@ -161,7 +161,7 @@ static int musl_getopt_long(
 	if (resumed > skipped) {
 		int i, cnt = musl_optind - resumed;
 
-		for (i = 0; i < cnt; i++) {
+		for (i = 0; i < cnt; ++i) {
 			permute(argv, skipped, musl_optind - 1);
 		}
 		musl_optind = skipped + cnt;
@@ -180,16 +180,16 @@ static int musl_getopt_long_core(
 		int i, cnt, match = 0;
 		char *arg = 0, *opt, *start = argv[musl_optind] + 1;
 
-		for (cnt = i = 0; longopts[i].name; i++) {
+		for (cnt = i = 0; longopts[i].name; ++i) {
 			char const *name = longopts[i].name;
 
 			opt = start;
 			if (*opt == '-') {
-				opt++;
+				++opt;
 			}
 			while (*opt && *opt != '=' && *opt == *name) {
-				name++;
-				opt++;
+				++name;
+				++opt;
 			}
 			if (*opt && *opt != '=') {
 				continue;
@@ -200,19 +200,19 @@ static int musl_getopt_long_core(
 				cnt = 1;
 				break;
 			}
-			cnt++;
+			++cnt;
 		}
 		if (cnt == 1 && longonly && arg - start == mblen(start, MB_LEN_MAX)) {
 			int l = arg - start;
 
-			for (i = 0; optstring[i]; i++) {
+			for (i = 0; optstring[i]; ++i) {
 				int j = 0;
 
 				while (j < l && start[j] == optstring[i + j]) {
-					j++;
+					++j;
 				}
 				if (j == l) {
-					cnt++;
+					++cnt;
 					break;
 				}
 			}
@@ -220,7 +220,7 @@ static int musl_getopt_long_core(
 		if (cnt == 1) {
 			i = match;
 			opt = arg;
-			musl_optind++;
+			++musl_optind;
 			if (*opt == '=') {
 				if (!longopts[i].has_arg) {
 					musl_optopt = longopts[i].val;
@@ -254,7 +254,7 @@ static int musl_getopt_long_core(
 					);
 					return '?';
 				}
-				musl_optind++;
+				++musl_optind;
 			}
 			if (idx) {
 				*idx = i;
@@ -275,7 +275,7 @@ static int musl_getopt_long_core(
 				    strlen(argv[musl_optind] + 2)
 				);
 			}
-			musl_optind++;
+			++musl_optind;
 			return '?';
 		}
 	}
