@@ -14,14 +14,17 @@
 #include <string_view>
 #include <vector>
 
+#include "diagnostics.hpp"
 #include "extern/getopt.hpp"
 #include "file.hpp"
 #include "platform.hpp"
+#include "usage.hpp"
 #include "version.hpp"
 
 #include "gfx/pal_spec.hpp"
 #include "gfx/process.hpp"
 #include "gfx/reverse.hpp"
+#include "gfx/warning.hpp"
 
 using namespace std::literals::string_view_literals;
 
@@ -37,69 +40,6 @@ static struct LocalOptions {
 	bool reverse;
 } localOptions;
 
-static uintmax_t nbErrors;
-
-[[noreturn]]
-void giveUp() {
-	fprintf(stderr, "Conversion aborted after %ju error%s\n", nbErrors, nbErrors == 1 ? "" : "s");
-	exit(1);
-}
-
-void requireZeroErrors() {
-	if (nbErrors != 0) {
-		giveUp();
-	}
-}
-
-void warning(char const *fmt, ...) {
-	va_list ap;
-
-	fputs("warning: ", stderr);
-	va_start(ap, fmt);
-	vfprintf(stderr, fmt, ap);
-	va_end(ap);
-	putc('\n', stderr);
-}
-
-void error(char const *fmt, ...) {
-	va_list ap;
-
-	fputs("error: ", stderr);
-	va_start(ap, fmt);
-	vfprintf(stderr, fmt, ap);
-	va_end(ap);
-	putc('\n', stderr);
-
-	if (nbErrors != std::numeric_limits<decltype(nbErrors)>::max()) {
-		nbErrors++;
-	}
-}
-
-void errorMessage(char const *msg) {
-	fprintf(stderr, "error: %s\n", msg);
-
-	if (nbErrors != std::numeric_limits<decltype(nbErrors)>::max()) {
-		nbErrors++;
-	}
-}
-
-[[noreturn]]
-void fatal(char const *fmt, ...) {
-	va_list ap;
-
-	fputs("FATAL: ", stderr);
-	va_start(ap, fmt);
-	vfprintf(stderr, fmt, ap);
-	va_end(ap);
-	putc('\n', stderr);
-
-	if (nbErrors != std::numeric_limits<decltype(nbErrors)>::max()) {
-		nbErrors++;
-	}
-
-	giveUp();
-}
-
 void Options::verbosePrint(uint8_t level, char const *fmt, ...) const {
 	// LCOV_EXCL_START
 	if (verbosity >= level) {
@@ -113,7 +53,7 @@ void Options::verbosePrint(uint8_t level, char const *fmt, ...) const {
 }
 
 // Short options
-static char const *optstring = "-Aa:B:b:Cc:d:hi:L:mN:n:Oo:Pp:Qq:r:s:Tt:U:uVvXx:YZ";
+static char const *optstring = "-Aa:B:b:Cc:d:hi:L:l:mN:n:Oo:Pp:Qq:r:s:Tt:U:uVvW:wXx:YZ";
 
 // Equivalent long options
 // Please keep in the same order as short opts.
@@ -133,6 +73,7 @@ static option const longopts[] = {
     {"help",             no_argument,       nullptr, 'h'},
     {"input-tileset",    required_argument, nullptr, 'i'},
     {"slice",            required_argument, nullptr, 'L'},
+    {"base-palette",     required_argument, nullptr, 'l'},
     {"mirror-tiles",     no_argument,       nullptr, 'm'},
     {"nb-tiles",         required_argument, nullptr, 'N'},
     {"nb-palettes",      required_argument, nullptr, 'n'},
@@ -150,6 +91,7 @@ static option const longopts[] = {
     {"unique-tiles",     no_argument,       nullptr, 'u'},
     {"version",          no_argument,       nullptr, 'V'},
     {"verbose",          no_argument,       nullptr, 'v'},
+    {"warning",          required_argument, nullptr, 'W'},
     {"mirror-x",         no_argument,       nullptr, 'X'},
     {"trim-end",         required_argument, nullptr, 'x'},
     {"mirror-y",         no_argument,       nullptr, 'Y'},
@@ -157,26 +99,23 @@ static option const longopts[] = {
     {nullptr,            no_argument,       nullptr, 0  }
 };
 
-// LCOV_EXCL_START
-static void printUsage() {
-	fputs(
-	    "Usage: rgbgfx [-r stride] [-ChmOuVXYZ] [-v [-v ...]] [-a <attr_map> | -A]\n"
-	    "       [-b <base_ids>] [-c <colors>] [-d <depth>] [-i <tileset_file>]\n"
-	    "       [-L <slice>] [-N <nb_tiles>] [-n <nb_pals>] [-o <out_file>]\n"
-	    "       [-p <pal_file> | -P] [-q <pal_map> | -Q] [-s <nb_colors>]\n"
-	    "       [-t <tile_map> | -T] [-x <nb_tiles>] <file>\n"
-	    "Useful options:\n"
-	    "    -m, --mirror-tiles    optimize out mirrored tiles\n"
-	    "    -o, --output <path>   output the tile data to this path\n"
-	    "    -t, --tilemap <path>  output the tile map to this path\n"
-	    "    -u, --unique-tiles    optimize out identical tiles\n"
-	    "    -V, --version         print RGBGFX version and exit\n"
-	    "\n"
-	    "For help, use `man rgbgfx' or go to https://rgbds.gbdev.io/docs/\n",
-	    stderr
-	);
-}
-// LCOV_EXCL_STOP
+// clang-format off: long string literal
+static Usage usage(
+    "Usage: rgbgfx [-r stride] [-ChmOuVXYZ] [-v [-v ...]] [-a <attr_map> | -A]\n"
+    "       [-b <base_ids>] [-c <colors>] [-d <depth>] [-i <tileset_file>]\n"
+    "       [-L <slice>] [-l <base_pal>] [-N <nb_tiles>] [-n <nb_pals>]\n"
+    "       [-o <out_file>] [-p <pal_file> | -P] [-q <pal_map> | -Q]\n"
+    "       [-s <nb_colors>] [-t <tile_map> | -T] [-x <nb_tiles>] <file>\n"
+    "Useful options:\n"
+    "    -m, --mirror-tiles    optimize out mirrored tiles\n"
+    "    -o, --output <path>   output the tile data to this path\n"
+    "    -t, --tilemap <path>  output the tile map to this path\n"
+    "    -u, --unique-tiles    optimize out identical tiles\n"
+    "    -V, --version         print RGBGFX version and exit\n"
+    "\n"
+    "For help, use `man rgbgfx' or go to https://rgbds.gbdev.io/docs/\n"
+);
+// clang-format on
 
 // Parses a number at the beginning of a string, moving the pointer to skip the parsed characters.
 // Returns the provided errVal on error.
@@ -255,19 +194,13 @@ static void skipWhitespace(char *&arg) {
 
 static void registerInput(char const *arg) {
 	if (!options.input.empty()) {
-		fprintf(
-		    stderr,
-		    "FATAL: input image specified more than once! (first \"%s\", then "
-		    "\"%s\")\n",
+		usage.printAndExit(
+		    "Input image specified more than once! (first \"%s\", then \"%s\")",
 		    options.input.c_str(),
 		    arg
 		);
-		printUsage();
-		exit(1);
 	} else if (arg[0] == '\0') { // Empty input path
-		fprintf(stderr, "FATAL: input image path cannot be empty\n");
-		printUsage();
-		exit(1);
+		usage.printAndExit("Input image path cannot be empty");
 	} else {
 		options.input = arg;
 	}
@@ -360,7 +293,7 @@ static char *parseArgv(int argc, char *argv[]) {
 		case 'a':
 			localOptions.autoAttrmap = false;
 			if (!options.attrmap.empty()) {
-				warning("Overriding attrmap file %s", options.attrmap.c_str());
+				warnx("Overriding attrmap file %s", options.attrmap.c_str());
 			}
 			options.attrmap = musl_optarg;
 			break;
@@ -431,13 +364,10 @@ static char *parseArgv(int argc, char *argv[]) {
 			}
 			break;
 		case 'h':
-			// LCOV_EXCL_START
-			printUsage();
-			exit(0);
-			// LCOV_EXCL_STOP
+			usage.printAndExit(0); // LCOV_EXCL_LINE
 		case 'i':
 			if (!options.inputTileset.empty()) {
-				warning("Overriding input tileset file %s", options.inputTileset.c_str());
+				warnx("Overriding input tileset file %s", options.inputTileset.c_str());
 			}
 			options.inputTileset = musl_optarg;
 			break;
@@ -481,6 +411,16 @@ static char *parseArgv(int argc, char *argv[]) {
 				error("Unexpected extra characters after slice spec in \"%s\"", musl_optarg);
 			}
 			break;
+		case 'l':
+			number = parseNumber(arg, "Base palette ID", 0);
+			if (*arg != '\0') {
+				error("Base palette ID must be a valid number, not \"%s\"", musl_optarg);
+			} else if (number >= 256) {
+				error("Base palette ID must be below 256");
+			} else {
+				options.basePalID = number;
+			}
+			break;
 		case 'm':
 			options.allowMirroringX = true; // Imply `-X`
 			options.allowMirroringY = true; // Imply `-Y`
@@ -537,7 +477,7 @@ static char *parseArgv(int argc, char *argv[]) {
 			break;
 		case 'o':
 			if (!options.output.empty()) {
-				warning("Overriding tile data file %s", options.output.c_str());
+				warnx("Overriding tile data file %s", options.output.c_str());
 			}
 			options.output = musl_optarg;
 			break;
@@ -547,7 +487,7 @@ static char *parseArgv(int argc, char *argv[]) {
 		case 'p':
 			localOptions.autoPalettes = false;
 			if (!options.palettes.empty()) {
-				warning("Overriding palettes file %s", options.palettes.c_str());
+				warnx("Overriding palettes file %s", options.palettes.c_str());
 			}
 			options.palettes = musl_optarg;
 			break;
@@ -557,7 +497,7 @@ static char *parseArgv(int argc, char *argv[]) {
 		case 'q':
 			localOptions.autoPalmap = false;
 			if (!options.palmap.empty()) {
-				warning("Overriding palette map file %s", options.palmap.c_str());
+				warnx("Overriding palette map file %s", options.palmap.c_str());
 			}
 			options.palmap = musl_optarg;
 			break;
@@ -585,7 +525,7 @@ static char *parseArgv(int argc, char *argv[]) {
 		case 't':
 			localOptions.autoTilemap = false;
 			if (!options.tilemap.empty()) {
-				warning("Overriding tilemap file %s", options.tilemap.c_str());
+				warnx("Overriding tilemap file %s", options.tilemap.c_str());
 			}
 			options.tilemap = musl_optarg;
 			break;
@@ -601,6 +541,12 @@ static char *parseArgv(int argc, char *argv[]) {
 			}
 			break;
 			// LCOV_EXCL_STOP
+		case 'W':
+			warnings.processWarningFlag(musl_optarg);
+			break;
+		case 'w':
+			warnings.state.warningsEnabled = false;
+			break;
 		case 'x':
 			options.trim = parseNumber(arg, "Number of tiles to trim", 0);
 			if (*arg != '\0') {
@@ -627,16 +573,173 @@ static char *parseArgv(int argc, char *argv[]) {
 			}
 			break;
 		default:
-			// LCOV_EXCL_START
-			printUsage();
-			exit(1);
-			// LCOV_EXCL_STOP
+			usage.printAndExit(1); // LCOV_EXCL_LINE
 		}
 	}
 
 	return nullptr; // Done processing this argv
 }
 
+static void verboseOutputConfig() {
+	fprintf(stderr, "rgbgfx %s\n", get_package_version_string());
+
+	if (options.verbosity >= Options::VERB_VVVVVV) {
+		putc('\n', stderr);
+		// clang-format off: vertically align values
+		static std::array<uint16_t, 21> gfx{
+		    0b0111111110,
+		    0b1111111111,
+		    0b1110011001,
+		    0b1110011001,
+		    0b1111111111,
+		    0b1111111111,
+		    0b1110000001,
+		    0b1111000011,
+		    0b0111111110,
+		    0b0001111000,
+		    0b0111111110,
+		    0b1111111111,
+		    0b1111111111,
+		    0b1111111111,
+		    0b1101111011,
+		    0b1101111011,
+		    0b0011111100,
+		    0b0011001100,
+		    0b0111001110,
+		    0b0111001110,
+		    0b0111001110,
+		};
+		// clang-format on
+		static std::array<char const *, 3> textbox{
+		    "  ,----------------------------------------.",
+		    "  | Augh, dimensional interference again?! |",
+		    "  `----------------------------------------'",
+		};
+		for (size_t i = 0; i < gfx.size(); ++i) {
+			uint16_t row = gfx[i];
+			for (uint8_t _ = 0; _ < 10; ++_) {
+				unsigned char c = row & 1 ? '0' : ' ';
+				putc(c, stderr);
+				// Double the pixel horizontally, otherwise the aspect ratio looks wrong
+				putc(c, stderr);
+				row >>= 1;
+			}
+			if (i < textbox.size()) {
+				fputs(textbox[i], stderr);
+			}
+			putc('\n', stderr);
+		}
+		putc('\n', stderr);
+	}
+
+	fputs("Options:\n", stderr);
+	if (options.columnMajor) {
+		fputs("\tVisit image in column-major order\n", stderr);
+	}
+	if (options.allowDedup) {
+		fputs("\tAllow deduplicating tiles\n", stderr);
+	}
+	if (options.allowMirroringX) {
+		fputs("\tAllow deduplicating horizontally mirrored tiles\n", stderr);
+	}
+	if (options.allowMirroringY) {
+		fputs("\tAllow deduplicating vertically mirrored tiles\n", stderr);
+	}
+	if (options.useColorCurve) {
+		fputs("\tUse color curve\n", stderr);
+	}
+	fprintf(stderr, "\tBit depth: %" PRIu8 "bpp\n", options.bitDepth);
+	if (options.trim != 0) {
+		fprintf(stderr, "\tTrim the last %" PRIu64 " tiles\n", options.trim);
+	}
+	fprintf(stderr, "\tMaximum %" PRIu16 " palettes\n", options.nbPalettes);
+	fprintf(stderr, "\tPalettes contain %" PRIu8 " colors\n", options.nbColorsPerPal);
+	fprintf(stderr, "\t%s palette spec\n", [] {
+		switch (options.palSpecType) {
+		case Options::NO_SPEC:
+			return "No";
+		case Options::EXPLICIT:
+			return "Explicit";
+		case Options::EMBEDDED:
+			return "Embedded";
+		case Options::DMG:
+			return "DMG";
+		}
+		return "???";
+	}());
+	if (options.palSpecType == Options::EXPLICIT) {
+		fputs("\t[\n", stderr);
+		for (auto const &pal : options.palSpec) {
+			fputs("\t\t", stderr);
+			for (auto const &color : pal) {
+				if (color) {
+					fprintf(stderr, "#%06x, ", color->toCSS() >> 8);
+				} else {
+					fputs("#none, ", stderr);
+				}
+			}
+			putc('\n', stderr);
+		}
+		fputs("\t]\n", stderr);
+	}
+	fprintf(
+	    stderr,
+	    "\tInput image slice: %" PRIu16 "x%" PRIu16 " pixels starting at (%" PRIu16 ", %" PRIu16
+	    ")\n",
+	    options.inputSlice.width,
+	    options.inputSlice.height,
+	    options.inputSlice.left,
+	    options.inputSlice.top
+	);
+	fprintf(
+	    stderr,
+	    "\tBase tile IDs: [%" PRIu8 ", %" PRIu8 "]\n",
+	    options.baseTileIDs[0],
+	    options.baseTileIDs[1]
+	);
+	fprintf(stderr, "\tBase palette ID: %" PRIu8 "\n", options.basePalID);
+	fprintf(
+	    stderr,
+	    "\tMaximum %" PRIu16 " tiles in bank 0, %" PRIu16 " in bank 1\n",
+	    options.maxNbTiles[0],
+	    options.maxNbTiles[1]
+	);
+	auto printPath = [](char const *name, std::string const &path) {
+		if (!path.empty()) {
+			fprintf(stderr, "\t%s: %s\n", name, path.c_str());
+		}
+	};
+	printPath("Input image", options.input);
+	printPath("Output tile data", options.output);
+	printPath("Output tilemap", options.tilemap);
+	printPath("Output attrmap", options.attrmap);
+	printPath("Output palettes", options.palettes);
+	fputs("Ready.\n", stderr);
+}
+
+// Manual implementation of std::filesystem::path.replace_extension().
+// macOS <10.15 did not support std::filesystem::path.
+static void replaceExtension(std::string &path, char const *extension) {
+	constexpr std::string_view chars =
+// Both must start with a dot!
+#if defined(_MSC_VER) || defined(__MINGW32__)
+	    "./\\"sv;
+#else
+	    "./"sv;
+#endif
+	size_t len = path.npos;
+	if (size_t i = path.find_last_of(chars); i != path.npos && path[i] == '.') {
+		// We found the last dot, but check if it's part of a stem
+		// (There must be a non-path separator character before it)
+		if (i != 0 && chars.find(path[i - 1], 1) == chars.npos) {
+			// We can replace the extension
+			len = i;
+		}
+	}
+	path.assign(path, 0, len);
+	path.append(extension);
+}
+
 int main(int argc, char *argv[]) {
 	struct AtFileStackEntry {
 		int parentInd;            // Saved offset into parent argv
@@ -656,14 +759,14 @@ int main(int argc, char *argv[]) {
 			// We need to allocate a new arg pool for each at-file, so as not to invalidate pointers
 			// previous at-files may have generated to their own arg pools.
 			// But for the same reason, the arg pool must also outlive the at-file's stack entry!
-			auto &argPool = argPools.emplace_back();
+			std::vector<char> &argPool = argPools.emplace_back();
 
 			// Copy `argv[0]` for error reporting, and because option parsing skips it
 			AtFileStackEntry &stackEntry =
 			    atFileStack.emplace_back(musl_optind, std::vector{atFileName});
 			// It would be nice to compute the char pointers on the fly, but reallocs don't allow
 			// that; so we must compute the offsets after the pool is fixed
-			auto offsets = readAtFile(&musl_optarg[1], argPool);
+			std::vector<size_t> offsets = readAtFile(&musl_optarg[1], argPool);
 			stackEntry.argv.reserve(offsets.size() + 2); // Avoid a bunch of reallocs
 			for (size_t ofs : offsets) {
 				stackEntry.argv.push_back(&argPool.data()[ofs]);
@@ -696,7 +799,7 @@ int main(int argc, char *argv[]) {
 			curArgc = argc;
 			curArgv = argv;
 		} else {
-			auto &vec = atFileStack.back().argv;
+			std::vector<char *> &vec = atFileStack.back().argv;
 			curArgc = vec.size();
 			curArgv = vec.data();
 		}
@@ -714,39 +817,17 @@ int main(int argc, char *argv[]) {
 	}
 
 	auto autoOutPath = [](bool autoOptEnabled, std::string &path, char const *extension) {
-		if (autoOptEnabled) {
-			auto &image = localOptions.groupOutputs ? options.output : options.input;
-			if (image.empty()) {
-				fprintf(
-				    stderr,
-				    "FATAL: No %s specified\n",
-				    localOptions.groupOutputs ? "output tile data file" : "input image"
-				);
-				printUsage();
-				exit(1);
-			}
-
-			// Manual implementation of std::filesystem::path.replace_extension().
-			constexpr std::string_view chars =
-// Both must start with a dot!
-#if defined(_MSC_VER) || defined(__MINGW32__)
-			    "./\\"sv;
-#else
-			    "./"sv;
-#endif
-			size_t len = image.npos;
-			size_t i = image.find_last_of(chars);
-			if (i != image.npos && image[i] == '.') {
-				// We found the last dot, but check if it's part of a stem
-				// (There must be a non-path separator character before it)
-				if (i != 0 && chars.find(image[i - 1], 1) == chars.npos) {
-					// We can replace the extension
-					len = i;
-				}
-			}
-			path.assign(image, 0, len);
-			path.append(extension);
+		if (!autoOptEnabled) {
+			return;
 		}
+		path = localOptions.groupOutputs ? options.output : options.input;
+		if (path.empty()) {
+			usage.printAndExit(
+			    "No %s specified",
+			    localOptions.groupOutputs ? "output tile data file" : "input image"
+			);
+		}
+		replaceExtension(path, extension);
 	};
 	autoOutPath(localOptions.autoAttrmap, options.attrmap, ".attrmap");
 	autoOutPath(localOptions.autoTilemap, options.tilemap, ".tilemap");
@@ -760,139 +841,7 @@ int main(int argc, char *argv[]) {
 
 	// LCOV_EXCL_START
 	if (options.verbosity >= Options::VERB_CFG) {
-		fprintf(stderr, "rgbgfx %s\n", get_package_version_string());
-
-		if (options.verbosity >= Options::VERB_VVVVVV) {
-			putc('\n', stderr);
-			// clang-format off: vertically align values
-			static std::array<uint16_t, 21> gfx{
-			    0b0111111110,
-			    0b1111111111,
-			    0b1110011001,
-			    0b1110011001,
-			    0b1111111111,
-			    0b1111111111,
-			    0b1110000001,
-			    0b1111000011,
-			    0b0111111110,
-			    0b0001111000,
-			    0b0111111110,
-			    0b1111111111,
-			    0b1111111111,
-			    0b1111111111,
-			    0b1101111011,
-			    0b1101111011,
-			    0b0011111100,
-			    0b0011001100,
-			    0b0111001110,
-			    0b0111001110,
-			    0b0111001110,
-			};
-			// clang-format on
-			static std::array<char const *, 3> textbox{
-			    "  ,----------------------------------------.",
-			    "  | Augh, dimensional interference again?! |",
-			    "  `----------------------------------------'",
-			};
-			for (size_t i = 0; i < gfx.size(); ++i) {
-				uint16_t row = gfx[i];
-				for (uint8_t _ = 0; _ < 10; ++_) {
-					unsigned char c = row & 1 ? '0' : ' ';
-					putc(c, stderr);
-					// Double the pixel horizontally, otherwise the aspect ratio looks wrong
-					putc(c, stderr);
-					row >>= 1;
-				}
-				if (i < textbox.size()) {
-					fputs(textbox[i], stderr);
-				}
-				putc('\n', stderr);
-			}
-			putc('\n', stderr);
-		}
-
-		fputs("Options:\n", stderr);
-		if (options.columnMajor) {
-			fputs("\tVisit image in column-major order\n", stderr);
-		}
-		if (options.allowDedup) {
-			fputs("\tAllow deduplicating tiles\n", stderr);
-		}
-		if (options.allowMirroringX) {
-			fputs("\tAllow deduplicating horizontally mirrored tiles\n", stderr);
-		}
-		if (options.allowMirroringY) {
-			fputs("\tAllow deduplicating vertically mirrored tiles\n", stderr);
-		}
-		if (options.useColorCurve) {
-			fputs("\tUse color curve\n", stderr);
-		}
-		fprintf(stderr, "\tBit depth: %" PRIu8 "bpp\n", options.bitDepth);
-		if (options.trim != 0) {
-			fprintf(stderr, "\tTrim the last %" PRIu64 " tiles\n", options.trim);
-		}
-		fprintf(stderr, "\tMaximum %" PRIu16 " palettes\n", options.nbPalettes);
-		fprintf(stderr, "\tPalettes contain %" PRIu8 " colors\n", options.nbColorsPerPal);
-		fprintf(stderr, "\t%s palette spec\n", [] {
-			switch (options.palSpecType) {
-			case Options::NO_SPEC:
-				return "No";
-			case Options::EXPLICIT:
-				return "Explicit";
-			case Options::EMBEDDED:
-				return "Embedded";
-			case Options::DMG:
-				return "DMG";
-			}
-			return "???";
-		}());
-		if (options.palSpecType == Options::EXPLICIT) {
-			fputs("\t[\n", stderr);
-			for (auto const &pal : options.palSpec) {
-				fputs("\t\t", stderr);
-				for (auto &color : pal) {
-					if (color) {
-						fprintf(stderr, "#%06x, ", color->toCSS() >> 8);
-					} else {
-						fputs("#none, ", stderr);
-					}
-				}
-				putc('\n', stderr);
-			}
-			fputs("\t]\n", stderr);
-		}
-		fprintf(
-		    stderr,
-		    "\tInput image slice: %" PRIu32 "x%" PRIu32 " pixels starting at (%" PRId32 ", %" PRId32
-		    ")\n",
-		    options.inputSlice.width,
-		    options.inputSlice.height,
-		    options.inputSlice.left,
-		    options.inputSlice.top
-		);
-		fprintf(
-		    stderr,
-		    "\tBase tile IDs: [%" PRIu8 ", %" PRIu8 "]\n",
-		    options.baseTileIDs[0],
-		    options.baseTileIDs[1]
-		);
-		fprintf(
-		    stderr,
-		    "\tMaximum %" PRIu16 " tiles in bank 0, %" PRIu16 " in bank 1\n",
-		    options.maxNbTiles[0],
-		    options.maxNbTiles[1]
-		);
-		auto printPath = [](char const *name, std::string const &path) {
-			if (!path.empty()) {
-				fprintf(stderr, "\t%s: %s\n", name, path.c_str());
-			}
-		};
-		printPath("Input image", options.input);
-		printPath("Output tile data", options.output);
-		printPath("Output tilemap", options.tilemap);
-		printPath("Output attrmap", options.attrmap);
-		printPath("Output palettes", options.palettes);
-		fputs("Ready.\n", stderr);
+		verboseOutputConfig();
 	}
 	// LCOV_EXCL_STOP
 
@@ -909,9 +858,7 @@ int main(int argc, char *argv[]) {
 	           && !localOptions.reverse) {
 		processPalettes();
 	} else {
-		fputs("FATAL: No input image specified\n", stderr);
-		printUsage();
-		exit(1);
+		usage.printAndExit("No input image specified");
 	}
 
 	requireZeroErrors();
