@@ -4,6 +4,7 @@
 
 #include <algorithm>
 #include <charconv>
+#include <errno.h>
 #include <fstream>
 #include <inttypes.h>
 #include <limits.h>
@@ -14,15 +15,21 @@
 #include <string.h>
 #include <string>
 #include <string_view>
-#include <tuple>
+#include <utility>
 
+#include "diagnostics.hpp"
 #include "helpers.hpp"
 #include "platform.hpp"
+#include "util.hpp" // UpperMap
 
 #include "gfx/main.hpp"
+#include "gfx/png.hpp"
+#include "gfx/warning.hpp"
 
 using namespace std::string_view_literals;
 
+static char const *hexDigits = "0123456789ABCDEFabcdef";
+
 template<typename Str> // Should be std::string or std::string_view
 static void skipWhitespace(Str const &str, size_t &pos) {
 	pos = std::min(str.find_first_not_of(" \t"sv, pos), str.length());
@@ -60,17 +67,17 @@ void parseInlinePalSpec(char const * const rawArg) {
 		assume(ofs <= arg.length());
 		assume(len <= arg.length());
 
-		errorMessage(msg);
+		error("%s", msg); // `format_` and `-Wformat-security` would complain about `error(msg);`
 		fprintf(
 		    stderr,
 		    "In inline palette spec: %s\n"
 		    "                        ",
 		    rawArg
 		);
-		for (auto i = ofs; i; --i) {
+		for (size_t i = ofs; i; --i) {
 			putc(' ', stderr);
 		}
-		for (auto i = len; i; --i) {
+		for (size_t i = len; i; --i) {
 			putc('^', stderr);
 		}
 		putc('\n', stderr);
@@ -90,7 +97,7 @@ void parseInlinePalSpec(char const * const rawArg) {
 			color = {};
 			n += literal_strlen("none");
 		} else {
-			auto pos = std::min(arg.find_first_not_of("0123456789ABCDEFabcdef"sv, n), arg.length());
+			size_t pos = std::min(arg.find_first_not_of(hexDigits, n), arg.length());
 			switch (pos - n) {
 			case 3:
 				color = Rgba(
@@ -193,7 +200,7 @@ static T readLE(U const *bytes) {
 static bool readLine(std::filebuf &file, std::string &buffer) {
 	assume(buffer.empty());
 	for (;;) {
-		auto c = file.sbumpc();
+		int c = file.sbumpc();
 		if (c == std::filebuf::traits_type::eof()) {
 			return !buffer.empty();
 		}
@@ -209,19 +216,32 @@ static bool readLine(std::filebuf &file, std::string &buffer) {
 	}
 }
 
-#define requireLine(kind, file, buffer) \
+#define requireLine(kind, filename, file, buffer) \
 	do { \
 		if (!readLine(file, buffer)) { \
-			error(kind " palette file is shorter than expected"); \
+			error(kind " palette file \"%s\" is shorter than expected", filename); \
 			return; \
 		} \
 	} while (0)
 
+static void warnExtraColors(
+    char const *kind, char const *filename, uint16_t nbColors, uint16_t maxNbColors
+) {
+	warnx(
+	    "%s file \"%s\" contains %" PRIu16 " colors, but there can only be %" PRIu16
+	    "; ignoring extra",
+	    kind,
+	    filename,
+	    nbColors,
+	    maxNbColors
+	);
+}
+
 // Parses the initial part of a string_view, advancing the "read index" as it does
 template<typename U> // Should be uint*_t
 static std::optional<U> parseDec(std::string const &str, size_t &n) {
 	uintmax_t value = 0;
-	auto result = std::from_chars(str.data() + n, str.data() + str.size(), value);
+	auto result = std::from_chars(str.data() + n, str.data() + str.length(), value);
 	if (static_cast<bool>(result.ec)) {
 		return std::nullopt;
 	}
@@ -259,24 +279,24 @@ static std::optional<Rgba> parseColor(std::string const &str, size_t &n, uint16_
 	return std::optional<Rgba>{Rgba(*r, *g, *b, 0xFF)};
 }
 
-static void parsePSPFile(std::filebuf &file) {
+static void parsePSPFile(char const *filename, std::filebuf &file) {
 	// https://www.selapa.net/swatches/colors/fileformats.php#psp_pal
 
 	std::string line;
 	if (!readLine(file, line) || line != "JASC-PAL") {
-		error("Palette file does not appear to be a PSP palette file");
+		error("File \"%s\" is not a valid PSP palette file", filename);
 		return;
 	}
 
 	line.clear();
-	requireLine("PSP", file, line);
+	requireLine("PSP", filename, file, line);
 	if (line != "0100") {
 		error("Unsupported PSP palette file version \"%s\"", line.c_str());
 		return;
 	}
 
 	line.clear();
-	requireLine("PSP", file, line);
+	requireLine("PSP", filename, file, line);
 	size_t n = 0;
 	std::optional<uint16_t> nbColors = parseDec<uint16_t>(line, n);
 	if (!nbColors || n != line.length()) {
@@ -284,22 +304,16 @@ static void parsePSPFile(std::filebuf &file) {
 		return;
 	}
 
-	if (uint16_t nbPalColors = options.nbColorsPerPal * options.nbPalettes;
-	    *nbColors > nbPalColors) {
-		warning(
-		    "PSP file contains %" PRIu16 " colors, but there can only be %" PRIu16
-		    "; ignoring extra",
-		    *nbColors,
-		    nbPalColors
-		);
-		nbColors = nbPalColors;
+	if (uint16_t maxNbColors = options.maxNbColors(); *nbColors > maxNbColors) {
+		warnExtraColors("PSP", filename, *nbColors, maxNbColors);
+		nbColors = maxNbColors;
 	}
 
 	options.palSpec.clear();
 
 	for (uint16_t i = 0; i < *nbColors; ++i) {
 		line.clear();
-		requireLine("PSP", file, line);
+		requireLine("PSP", filename, file, line);
 
 		n = 0;
 		std::optional<Rgba> color = parseColor(line, n, i + 1);
@@ -322,17 +336,17 @@ static void parsePSPFile(std::filebuf &file) {
 	}
 }
 
-static void parseGPLFile(std::filebuf &file) {
+static void parseGPLFile(char const *filename, std::filebuf &file) {
 	// https://gitlab.gnome.org/GNOME/gimp/-/blob/gimp-2-10/app/core/gimppalette-load.c#L39
 
 	std::string line;
 	if (!readLine(file, line) || !line.starts_with("GIMP Palette")) {
-		error("Palette file does not appear to be a GPL palette file");
+		error("File \"%s\" is not a valid GPL palette file", filename);
 		return;
 	}
 
 	uint16_t nbColors = 0;
-	uint16_t const maxNbColors = options.nbColorsPerPal * options.nbPalettes;
+	uint16_t const maxNbColors = options.maxNbColors();
 
 	for (;;) {
 		line.clear();
@@ -368,20 +382,15 @@ static void parseGPLFile(std::filebuf &file) {
 	}
 
 	if (nbColors > maxNbColors) {
-		warning(
-		    "GPL file contains %" PRIu16 " colors, but there can only be %" PRIu16
-		    "; ignoring extra",
-		    nbColors,
-		    maxNbColors
-		);
+		warnExtraColors("GPL", filename, nbColors, maxNbColors);
 	}
 }
 
-static void parseHEXFile(std::filebuf &file) {
+static void parseHEXFile(char const *filename, std::filebuf &file) {
 	// https://lospec.com/palette-list/tag/gbc
 
 	uint16_t nbColors = 0;
-	uint16_t const maxNbColors = options.nbColorsPerPal * options.nbPalettes;
+	uint16_t const maxNbColors = options.maxNbColors();
 
 	for (;;) {
 		std::string line;
@@ -393,8 +402,7 @@ static void parseHEXFile(std::filebuf &file) {
 			continue;
 		}
 
-		if (line.length() != 6
-		    || line.find_first_not_of("0123456789ABCDEFabcdef"sv) != std::string::npos) {
+		if (line.length() != 6 || line.find_first_not_of(hexDigits) != std::string::npos) {
 			error(
 			    "Failed to parse color #%d (\"%s\"): invalid \"rrggbb\" line",
 			    nbColors + 1,
@@ -416,42 +424,35 @@ static void parseHEXFile(std::filebuf &file) {
 	}
 
 	if (nbColors > maxNbColors) {
-		warning(
-		    "HEX file contains %" PRIu16 " colors, but there can only be %" PRIu16
-		    "; ignoring extra",
-		    nbColors,
-		    maxNbColors
-		);
+		warnExtraColors("HEX", filename, nbColors, maxNbColors);
 	}
 }
 
-static void parseACTFile(std::filebuf &file) {
+static void parseACTFile(char const *filename, std::filebuf &file) {
 	// https://www.adobe.com/devnet-apps/photoshop/fileformatashtml/#50577411_pgfId-1070626
 
 	std::array<char, 772> buf{};
-	auto len = file.sgetn(buf.data(), buf.size());
+	size_t len = file.sgetn(buf.data(), buf.size());
 
 	uint16_t nbColors = 256;
 	if (len == 772) {
 		nbColors = readBE<uint16_t>(&buf[768]);
 		if (nbColors > 256 || nbColors == 0) {
-			error("Invalid number of colors in ACT file (%" PRIu16 ")", nbColors);
+			error("Invalid number of colors in ACT file \"%s\" (%" PRIu16 ")", filename, nbColors);
 			return;
 		}
 	} else if (len != 768) {
-		error("Invalid file size for ACT file (expected 768 or 772 bytes, got %zu", len);
+		error(
+		    "Invalid file size for ACT file \"%s\" (expected 768 or 772 bytes, got %zu)",
+		    filename,
+		    len
+		);
 		return;
 	}
 
-	if (uint16_t nbPalColors = options.nbColorsPerPal * options.nbPalettes;
-	    nbColors > nbPalColors) {
-		warning(
-		    "ACT file contains %" PRIu16 " colors, but there can only be %" PRIu16
-		    "; ignoring extra",
-		    nbColors,
-		    nbPalColors
-		);
-		nbColors = nbPalColors;
+	if (uint16_t maxNbColors = options.maxNbColors(); nbColors > maxNbColors) {
+		warnExtraColors("ACT", filename, nbColors, maxNbColors);
+		nbColors = maxNbColors;
 	}
 
 	options.palSpec.clear();
@@ -477,7 +478,7 @@ static void parseACTFile(std::filebuf &file) {
 	}
 }
 
-static void parseACOFile(std::filebuf &file) {
+static void parseACOFile(char const *filename, std::filebuf &file) {
 	// https://www.adobe.com/devnet-apps/photoshop/fileformatashtml/#50577411_pgfId-1055819
 
 	char buf[10];
@@ -487,7 +488,7 @@ static void parseACOFile(std::filebuf &file) {
 		return;
 	}
 	if (readBE<uint16_t>(buf) != 1) {
-		error("Palette file does not appear to be an ACO v1 file");
+		error("File \"%s\" is not a valid ACO v1 file", filename);
 		return;
 	}
 
@@ -497,15 +498,9 @@ static void parseACOFile(std::filebuf &file) {
 	}
 	uint16_t nbColors = readBE<uint16_t>(buf);
 
-	if (uint16_t nbPalColors = options.nbColorsPerPal * options.nbPalettes;
-	    nbColors > nbPalColors) {
-		warning(
-		    "ACO file contains %" PRIu16 " colors, but there can only be %" PRIu16
-		    "; ignoring extra",
-		    nbColors,
-		    nbPalColors
-		);
-		nbColors = nbPalColors;
+	if (uint16_t maxNbColors = options.maxNbColors(); nbColors > maxNbColors) {
+		warnExtraColors("ACO", filename, nbColors, maxNbColors);
+		nbColors = maxNbColors;
 	}
 
 	options.palSpec.clear();
@@ -546,18 +541,18 @@ static void parseACOFile(std::filebuf &file) {
 	}
 }
 
-static void parseGBCFile(std::filebuf &file) {
+static void parseGBCFile(char const *filename, std::filebuf &file) {
 	// This only needs to be able to read back files generated by `rgbgfx -p`
 	options.palSpec.clear();
 
 	for (;;) {
 		char buf[2 * 4];
-		auto len = file.sgetn(buf, sizeof(buf));
-		if (len == 0) {
+		if (size_t len = file.sgetn(buf, sizeof(buf)); len == 0) {
 			break;
 		} else if (len != sizeof(buf)) {
 			error(
-			    "GBC palette dump contains %zu 8-byte palette%s, plus %zu byte%s",
+			    "GBC palette file \"%s\" contains %zu 8-byte palette%s, plus %zu byte%s",
+			    filename,
 			    options.palSpec.size(),
 			    options.palSpec.size() == 1 ? "" : "s",
 			    len,
@@ -575,6 +570,81 @@ static void parseGBCFile(std::filebuf &file) {
 	}
 }
 
+static bool checkPngSwatch(std::vector<Rgba> const &pixels, uint32_t base, uint32_t swatchSize) {
+	for (uint32_t y = 0; y < swatchSize; ++y) {
+		uint32_t yOffset = y * swatchSize * options.nbColorsPerPal + base;
+
+		for (uint32_t x = 0; x < swatchSize; ++x) {
+			if (x == 0 && y == 0) {
+				continue;
+			}
+			if (pixels[yOffset + x] != pixels[base]) {
+				return false;
+			}
+		}
+	}
+	return true;
+}
+
+static void parsePNGFile(char const *filename, std::filebuf &file) {
+	Png png{filename, file};
+
+	// The image width must evenly divide into a color swatch for each color per palette
+	if (png.width % options.nbColorsPerPal != 0) {
+		error(
+		    "PNG palette file is %" PRIu32 "x%" PRIu32 ", which is not a multiple of %" PRIu8
+		    " color swatches wide",
+		    png.width,
+		    png.height,
+		    options.nbColorsPerPal
+		);
+		return;
+	}
+
+	// Infer the color swatch size (width and height) from the image width
+	uint32_t swatchSize = png.width / options.nbColorsPerPal;
+
+	// The image height must evenly divide into a color swatch for each palette
+	if (png.height % swatchSize != 0) {
+		error(
+		    "PNG palette file is %" PRIu32 "x%" PRIu32 ", which is not a multiple of %" PRIu32
+		    " pixels high",
+		    png.width,
+		    png.height,
+		    swatchSize
+		);
+		return;
+	}
+
+	// More palettes than the maximum are a warning, not an error
+	uint32_t nbPals = png.height / swatchSize;
+	if (nbPals > options.nbPalettes) {
+		warnExtraColors(
+		    "PNG palette", filename, nbPals * options.nbColorsPerPal, options.maxNbColors()
+		);
+		nbPals = options.nbPalettes;
+	}
+
+	options.palSpec.clear();
+
+	// Get each color from the top-left pixel of each swatch
+	for (uint32_t y = 0; y < nbPals; ++y) {
+		uint32_t yOffset = y * swatchSize * swatchSize * options.nbColorsPerPal;
+		options.palSpec.emplace_back();
+
+		for (uint32_t x = 0; x < options.nbColorsPerPal; ++x) {
+			uint32_t offset = yOffset + x * swatchSize;
+			options.palSpec.back()[x] = png.pixels[offset];
+
+			// Check that each swatch is completely one color
+			if (!checkPngSwatch(png.pixels, offset, swatchSize)) {
+				error("PNG palette file uses multiple colors in one color swatch");
+				return;
+			}
+		}
+	}
+}
+
 void parseExternalPalSpec(char const *arg) {
 	// `fmt:path`, parse the file according to the given format
 
@@ -586,35 +656,30 @@ void parseExternalPalSpec(char const *arg) {
 	}
 	char const *path = ptr + 1;
 
-	static std::array parsers{
-	    std::tuple{"PSP", &parsePSPFile, std::ios::in    },
-	    std::tuple{"GPL", &parseGPLFile, std::ios::in    },
-	    std::tuple{"HEX", &parseHEXFile, std::ios::in    },
-	    std::tuple{"ACT", &parseACTFile, std::ios::binary},
-	    std::tuple{"ACO", &parseACOFile, std::ios::binary},
-	    std::tuple{"GBC", &parseGBCFile, std::ios::binary},
+	static UpperMap<std::pair<void (*)(char const *, std::filebuf &), bool>> const parsers{
+	    {"PSP", std::pair{&parsePSPFile, false}},
+	    {"GPL", std::pair{&parseGPLFile, false}},
+	    {"HEX", std::pair{&parseHEXFile, false}},
+	    {"ACT", std::pair{&parseACTFile, true} },
+	    {"ACO", std::pair{&parseACOFile, true} },
+	    {"GBC", std::pair{&parseGBCFile, true} },
+	    {"PNG", std::pair{&parsePNGFile, true} },
 	};
-
-	auto iter = std::find_if(RANGE(parsers), [&arg, &ptr](auto const &parser) {
-		return strncasecmp(arg, std::get<0>(parser), ptr - arg) == 0;
-	});
-	if (iter == parsers.end()) {
-		error(
-		    "Unknown external palette format \"%.*s\"",
-		    static_cast<int>(std::min(ptr - arg, static_cast<decltype(ptr - arg)>(INT_MAX))),
-		    arg
-		);
+	std::string format{arg, ptr};
+	auto search = parsers.find(format);
+	if (search == parsers.end()) {
+		error("Unknown external palette format \"%s\"", format.c_str());
 		return;
 	}
 
 	std::filebuf file;
 	// Some parsers read the file in text mode, others in binary mode
-	if (!file.open(path, std::ios::in | std::get<2>(*iter))) {
-		error("Failed to open palette file \"%s\"", path);
+	if (!file.open(path, search->second.second ? std::ios::in | std::ios::binary : std::ios::in)) {
+		fatal("Failed to open palette file \"%s\": %s", path, strerror(errno));
 		return;
 	}
 
-	std::get<1> (*iter)(file);
+	search->second.first(path, file);
 }
 
 void parseDmgPalSpec(char const * const rawArg) {
@@ -622,8 +687,7 @@ void parseDmgPalSpec(char const * const rawArg) {
 
 	std::string_view arg(rawArg);
 
-	if (arg.length() != 2
-	    || arg.find_first_not_of("0123456789ABCDEFabcdef"sv) != std::string_view::npos) {
+	if (arg.length() != 2 || arg.find_first_not_of(hexDigits) != std::string_view::npos) {
 		error("Unknown DMG palette specification \"%s\"", rawArg);
 		return;
 	}
@@ -657,7 +721,7 @@ void parseBackgroundPalSpec(char const *arg) {
 		return;
 	}
 
-	size_t size = strspn(&arg[1], "0123456789ABCDEFabcdef");
+	size_t size = strspn(&arg[1], hexDigits);
 	switch (size) {
 	case 3:
 		options.bgColor = Rgba(singleToHex(arg[1]), singleToHex(arg[2]), singleToHex(arg[3]), 0xFF);
