@@ -13,7 +13,7 @@
 #include <string.h>
 #include <vector>
 
-#include "error.hpp"
+#include "diagnostics.hpp"
 #include "helpers.hpp"
 #include "linkdefs.hpp"
 #include "platform.hpp"
@@ -21,9 +21,11 @@
 
 #include "link/assign.hpp"
 #include "link/main.hpp"
+#include "link/patch.hpp"
 #include "link/sdas_obj.hpp"
 #include "link/section.hpp"
 #include "link/symbol.hpp"
+#include "link/warning.hpp"
 
 static std::deque<std::vector<Symbol>> symbolLists;
 static std::vector<std::vector<FileStackNode>> nodes;
@@ -36,7 +38,7 @@ static std::vector<std::vector<FileStackNode>> nodes;
 		FILE *tmpFile = file; \
 		type tmpVal = func(tmpFile); \
 		if (tmpVal == (errval)) { \
-			errx(__VA_ARGS__, feof(tmpFile) ? "Unexpected end of file" : strerror(errno)); \
+			fatal(__VA_ARGS__, feof(tmpFile) ? "Unexpected end of file" : strerror(errno)); \
 		} \
 		var = static_cast<vartype>(tmpVal); \
 	} while (0)
@@ -75,9 +77,9 @@ static int64_t readLong(FILE *file) {
 	do { \
 		FILE *tmpFile = file; \
 		std::string &tmpVal = var; \
-		for (int tmpByte = getc(tmpFile); tmpByte != '\0'; tmpByte = getc(tmpFile)) { \
+		for (int tmpByte; (tmpByte = getc(tmpFile)) != '\0';) { \
 			if (tmpByte == EOF) { \
-				errx(__VA_ARGS__, feof(tmpFile) ? "Unexpected end of file" : strerror(errno)); \
+				fatal(__VA_ARGS__, feof(tmpFile) ? "Unexpected end of file" : strerror(errno)); \
 			} else { \
 				tmpVal.push_back(tmpByte); \
 			} \
@@ -91,12 +93,13 @@ static void readFileStackNode(
     FILE *file, std::vector<FileStackNode> &fileNodes, uint32_t nodeID, char const *fileName
 ) {
 	FileStackNode &node = fileNodes[nodeID];
-	uint32_t parentID;
 
+	uint32_t parentID;
 	tryReadLong(
 	    parentID, file, "%s: Cannot read node #%" PRIu32 "'s parent ID: %s", fileName, nodeID
 	);
 	node.parent = parentID != UINT32_MAX ? &fileNodes[parentID] : nullptr;
+
 	tryReadLong(
 	    node.lineNo, file, "%s: Cannot read node #%" PRIu32 "'s line number: %s", fileName, nodeID
 	);
@@ -123,7 +126,7 @@ static void readFileStackNode(
 		    depth, file, "%s: Cannot read node #%" PRIu32 "'s rept depth: %s", fileName, nodeID
 		);
 		node.data = std::vector<uint32_t>(depth);
-		for (uint32_t i = 0; i < depth; i++) {
+		for (uint32_t i = 0; i < depth; ++i) {
 			tryReadLong(
 			    node.iters()[i],
 			    file,
@@ -135,8 +138,6 @@ static void readFileStackNode(
 		}
 		if (!node.parent) {
 			fatal(
-			    nullptr,
-			    0,
 			    "%s is not a valid object file: root node (#%" PRIu32 ") may not be REPT",
 			    fileName,
 			    nodeID
@@ -207,9 +208,7 @@ static void readPatch(
     uint32_t patchID,
     std::vector<FileStackNode> const &fileNodes
 ) {
-	uint32_t nodeID, rpnSize;
-	PatchType type;
-
+	uint32_t nodeID;
 	tryReadLong(
 	    nodeID,
 	    file,
@@ -219,6 +218,7 @@ static void readPatch(
 	    patchID
 	);
 	patch.src = &fileNodes[nodeID];
+
 	tryReadLong(
 	    patch.lineNo,
 	    file,
@@ -251,6 +251,8 @@ static void readPatch(
 	    sectName.c_str(),
 	    patchID
 	);
+
+	PatchType type;
 	tryGetc(
 	    PatchType,
 	    type,
@@ -261,6 +263,8 @@ static void readPatch(
 	    patchID
 	);
 	patch.type = type;
+
+	uint32_t rpnSize;
 	tryReadLong(
 	    rpnSize,
 	    file,
@@ -271,10 +275,8 @@ static void readPatch(
 	);
 
 	patch.rpnExpression.resize(rpnSize);
-	size_t nbElementsRead = fread(patch.rpnExpression.data(), 1, rpnSize, file);
-
-	if (nbElementsRead != rpnSize) {
-		errx(
+	if (fread(patch.rpnExpression.data(), 1, rpnSize, file) != rpnSize) {
+		fatal(
 		    "%s: Cannot read \"%s\"'s patch #%" PRIu32 "'s RPN expression: %s",
 		    fileName,
 		    sectName.c_str(),
@@ -313,7 +315,7 @@ static void readSection(
 	);
 	tryReadLong(tmp, file, "%s: Cannot read \"%s\"'s' size: %s", fileName, section.name.c_str());
 	if (tmp < 0 || tmp > UINT16_MAX) {
-		errx("\"%s\"'s section size ($%" PRIx32 ") is invalid", section.name.c_str(), tmp);
+		fatal("\"%s\"'s section size ($%" PRIx32 ") is invalid", section.name.c_str(), tmp);
 	}
 	section.size = tmp;
 	section.offset = 0;
@@ -321,7 +323,7 @@ static void readSection(
 	    uint8_t, byte, file, "%s: Cannot read \"%s\"'s type: %s", fileName, section.name.c_str()
 	);
 	if (uint8_t type = byte & 0x3F; type >= SECTTYPE_INVALID) {
-		errx("\"%s\" has unknown section type 0x%02x", section.name.c_str(), type);
+		fatal("\"%s\" has unknown section type 0x%02x", section.name.c_str(), type);
 	} else {
 		section.type = SectionType(type);
 	}
@@ -335,7 +337,7 @@ static void readSection(
 	tryReadLong(tmp, file, "%s: Cannot read \"%s\"'s org: %s", fileName, section.name.c_str());
 	section.isAddressFixed = tmp >= 0;
 	if (tmp > UINT16_MAX) {
-		error(nullptr, 0, "\"%s\"'s org is too large ($%" PRIx32 ")", section.name.c_str(), tmp);
+		error("\"%s\"'s org is too large ($%" PRIx32 ")", section.name.c_str(), tmp);
 		tmp = UINT16_MAX;
 	}
 	section.org = tmp;
@@ -359,13 +361,7 @@ static void readSection(
 	    tmp, file, "%s: Cannot read \"%s\"'s alignment offset: %s", fileName, section.name.c_str()
 	);
 	if (tmp > UINT16_MAX) {
-		error(
-		    nullptr,
-		    0,
-		    "\"%s\"'s alignment offset is too large ($%" PRIx32 ")",
-		    section.name.c_str(),
-		    tmp
-		);
+		error("\"%s\"'s alignment offset is too large ($%" PRIx32 ")", section.name.c_str(), tmp);
 		tmp = UINT16_MAX;
 	}
 	section.alignOfs = tmp;
@@ -373,9 +369,8 @@ static void readSection(
 	if (sect_HasData(section.type)) {
 		if (section.size) {
 			section.data.resize(section.size);
-			if (size_t nbRead = fread(section.data.data(), 1, section.size, file);
-			    nbRead != section.size) {
-				errx(
+			if (fread(section.data.data(), 1, section.size, file) != section.size) {
+				fatal(
 				    "%s: Cannot read \"%s\"'s data: %s",
 				    fileName,
 				    section.name.c_str(),
@@ -385,7 +380,6 @@ static void readSection(
 		}
 
 		uint32_t nbPatches;
-
 		tryReadLong(
 		    nbPatches,
 		    file,
@@ -395,7 +389,7 @@ static void readSection(
 		);
 
 		section.patches.resize(nbPatches);
-		for (uint32_t i = 0; i < nbPatches; i++) {
+		for (uint32_t i = 0; i < nbPatches; ++i) {
 			readPatch(file, section.patches[i], fileName, section.name, i, fileNodes);
 		}
 	}
@@ -445,7 +439,7 @@ void obj_ReadFile(char const *fileName, unsigned int fileID) {
 		file = stdin;
 	}
 	if (!file) {
-		err("Failed to open file \"%s\"", fileName);
+		fatal("Failed to open file \"%s\": %s", fileName, strerror(errno));
 	}
 	Defer closeFile{[&] { fclose(file); }};
 
@@ -456,7 +450,7 @@ void obj_ReadFile(char const *fileName, unsigned int fileID) {
 	ungetc(c, file); // Guaranteed to work
 	switch (c) {
 	case EOF:
-		fatal(nullptr, 0, "File \"%s\" is empty!", fileName);
+		fatal("File \"%s\" is empty!", fileName);
 
 	case 'R':
 		break;
@@ -467,7 +461,7 @@ void obj_ReadFile(char const *fileName, unsigned int fileID) {
 		// object file. It's better than nothing.
 		nodes[fileID].push_back({
 		    .type = NODE_FILE,
-		    .data = Either<std::vector<uint32_t>, std::string>(fileName),
+		    .data = std::variant<std::monostate, std::vector<uint32_t>, std::string>(fileName),
 		    .parent = nullptr,
 		    .lineNo = 0,
 		});
@@ -483,16 +477,15 @@ void obj_ReadFile(char const *fileName, unsigned int fileID) {
 
 	if (fscanf(file, RGBDS_OBJECT_VERSION_STRING "%n", &matchedElems) == 1
 	    && matchedElems != literal_strlen(RGBDS_OBJECT_VERSION_STRING)) {
-		errx("%s: Not a RGBDS object file", fileName);
+		fatal("%s: Not a RGBDS object file", fileName);
 	}
 
 	verbosePrint("Reading object file %s\n", fileName);
 
 	uint32_t revNum;
-
 	tryReadLong(revNum, file, "%s: Cannot read revision number: %s", fileName);
 	if (revNum != RGBDS_OBJECT_REV) {
-		errx(
+		fatal(
 		    "%s: Unsupported object file for rgblink %s; try rebuilding \"%s\"%s"
 		    " (expected revision %d, got %d)",
 		    fileName,
@@ -504,15 +497,13 @@ void obj_ReadFile(char const *fileName, unsigned int fileID) {
 		);
 	}
 
-	uint32_t nbNodes;
 	uint32_t nbSymbols;
-	uint32_t nbSections;
-
 	tryReadLong(nbSymbols, file, "%s: Cannot read number of symbols: %s", fileName);
+
+	uint32_t nbSections;
 	tryReadLong(nbSections, file, "%s: Cannot read number of sections: %s", fileName);
 
-	nbSectionsToAssign += nbSections;
-
+	uint32_t nbNodes;
 	tryReadLong(nbNodes, file, "%s: Cannot read number of nodes: %s", fileName);
 	nodes[fileID].resize(nbNodes);
 	verbosePrint("Reading %u nodes...\n", nbNodes);
@@ -525,15 +516,15 @@ void obj_ReadFile(char const *fileName, unsigned int fileID) {
 	std::vector<uint32_t> nbSymPerSect(nbSections, 0);
 
 	verbosePrint("Reading %" PRIu32 " symbols...\n", nbSymbols);
-	for (uint32_t i = 0; i < nbSymbols; i++) {
+	for (uint32_t i = 0; i < nbSymbols; ++i) {
 		// Read symbol
 		Symbol &symbol = fileSymbols[i];
 
 		readSymbol(file, symbol, fileName, nodes[fileID]);
 
 		sym_AddSymbol(symbol);
-		if (symbol.data.holds<Label>()) {
-			nbSymPerSect[symbol.data.get<Label>().sectionID]++;
+		if (std::holds_alternative<Label>(symbol.data)) {
+			++nbSymPerSect[std::get<Label>(symbol.data).sectionID];
 		}
 	}
 
@@ -541,7 +532,7 @@ void obj_ReadFile(char const *fileName, unsigned int fileID) {
 	std::vector<std::unique_ptr<Section>> fileSections(nbSections);
 
 	verbosePrint("Reading %" PRIu32 " sections...\n", nbSections);
-	for (uint32_t i = 0; i < nbSections; i++) {
+	for (uint32_t i = 0; i < nbSections; ++i) {
 		// Read section
 		fileSections[i] = std::make_unique<Section>();
 		fileSections[i]->nextu = nullptr;
@@ -551,11 +542,10 @@ void obj_ReadFile(char const *fileName, unsigned int fileID) {
 	}
 
 	uint32_t nbAsserts;
-
 	tryReadLong(nbAsserts, file, "%s: Cannot read number of assertions: %s", fileName);
 	verbosePrint("Reading %" PRIu32 " assertions...\n", nbAsserts);
-	for (uint32_t i = 0; i < nbAsserts; i++) {
-		Assertion &assertion = assertions.emplace_front();
+	for (uint32_t i = 0; i < nbAsserts; ++i) {
+		Assertion &assertion = patch_AddAssertion();
 
 		readAssertion(file, assertion, fileName, i, nodes[fileID]);
 		linkPatchToPCSect(assertion.patch, fileSections);
@@ -563,7 +553,7 @@ void obj_ReadFile(char const *fileName, unsigned int fileID) {
 	}
 
 	// Give patches' PC section pointers to their sections
-	for (uint32_t i = 0; i < nbSections; i++) {
+	for (uint32_t i = 0; i < nbSections; ++i) {
 		if (sect_HasData(fileSections[i]->type)) {
 			for (Patch &patch : fileSections[i]->patches) {
 				linkPatchToPCSect(patch, fileSections);
@@ -572,9 +562,9 @@ void obj_ReadFile(char const *fileName, unsigned int fileID) {
 	}
 
 	// Give symbols' section pointers to their sections
-	for (uint32_t i = 0; i < nbSymbols; i++) {
-		if (fileSymbols[i].data.holds<Label>()) {
-			Label &label = fileSymbols[i].data.get<Label>();
+	for (uint32_t i = 0; i < nbSymbols; ++i) {
+		if (std::holds_alternative<Label>(fileSymbols[i].data)) {
+			Label &label = std::get<Label>(fileSymbols[i].data);
 			label.section = fileSections[label.sectionID].get();
 			// Give the section a pointer to the symbol as well
 			linkSymToSect(fileSymbols[i], *label.section);
@@ -582,25 +572,26 @@ void obj_ReadFile(char const *fileName, unsigned int fileID) {
 	}
 
 	// Calling `sect_AddSection` invalidates the contents of `fileSections`!
-	for (uint32_t i = 0; i < nbSections; i++) {
+	for (uint32_t i = 0; i < nbSections; ++i) {
 		sect_AddSection(std::move(fileSections[i]));
 	}
 
 	// Fix symbols' section pointers to component sections
 	// This has to run **after** all the `sect_AddSection()` calls,
 	// so that `sect_GetSection()` will work
-	for (uint32_t i = 0; i < nbSymbols; i++) {
-		if (fileSymbols[i].data.holds<Label>()) {
-			Label &label = fileSymbols[i].data.get<Label>();
-			if (Section *section = label.section; section->modifier != SECTION_NORMAL) {
-				if (section->modifier == SECTION_FRAGMENT) {
-					// Add the fragment's offset to the symbol's
-					// (`section->offset` is computed by `sect_AddSection`)
-					label.offset += section->offset;
-				}
+	for (uint32_t i = 0; i < nbSymbols; ++i) {
+		if (std::holds_alternative<Label>(fileSymbols[i].data)) {
+			Label &label = std::get<Label>(fileSymbols[i].data);
+			Section *section = label.section;
+			if (section->modifier != SECTION_NORMAL) {
 				// Associate the symbol with the main section, not the "component" one
 				label.section = sect_GetSection(section->name);
 			}
+			if (section->modifier == SECTION_FRAGMENT) {
+				// Add the fragment's offset to the symbol's
+				// (`section->offset` is computed by `sect_AddSection`)
+				label.offset += section->offset;
+			}
 		}
 	}
 }
