@@ -4,13 +4,14 @@
 
 #include <algorithm>
 #include <deque>
+#include <errno.h>
 #include <inttypes.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <vector>
 
-#include "error.hpp"
+#include "diagnostics.hpp"
 #include "extern/utf8decoder.hpp"
 #include "helpers.hpp"
 #include "linkdefs.hpp"
@@ -20,13 +21,14 @@
 #include "link/main.hpp"
 #include "link/section.hpp"
 #include "link/symbol.hpp"
+#include "link/warning.hpp"
 
 static constexpr size_t BANK_SIZE = 0x4000;
 
-FILE *outputFile;
-FILE *overlayFile;
-FILE *symFile;
-FILE *mapFile;
+static FILE *outputFile;
+static FILE *overlayFile;
+static FILE *symFile;
+static FILE *mapFile;
 
 struct SortedSymbol {
 	Symbol const *sym;
@@ -69,7 +71,7 @@ void out_AddSection(Section const &section) {
 	uint32_t minNbBanks = targetBank + 1;
 
 	if (minNbBanks > maxNbBanks[section.type]) {
-		errx(
+		fatal(
 		    "Section \"%s\" has an invalid bank range (%" PRIu32 " > %" PRIu32 ")",
 		    section.name.c_str(),
 		    section.bank,
@@ -77,19 +79,19 @@ void out_AddSection(Section const &section) {
 		);
 	}
 
-	for (uint32_t i = sections[section.type].size(); i < minNbBanks; i++) {
+	for (uint32_t i = sections[section.type].size(); i < minNbBanks; ++i) {
 		sections[section.type].emplace_back();
 	}
 
 	std::deque<Section const *> &bankSections =
 	    section.size ? sections[section.type][targetBank].sections
 	                 : sections[section.type][targetBank].zeroLenSections;
+
+	// Insert section while keeping the list sorted by increasing org
 	auto pos = bankSections.begin();
-
 	while (pos != bankSections.end() && (*pos)->org < section.org) {
-		pos++;
+		++pos;
 	}
-
 	bankSections.insert(pos, &section);
 }
 
@@ -123,7 +125,7 @@ static uint32_t checkOverlaySize() {
 
 	if (overlaySize % BANK_SIZE) {
 		warnx("Overlay file does not have a size multiple of 0x4000");
-	} else if (is32kMode && overlaySize != 0x8000) {
+	} else if (options.is32kMode && overlaySize != 0x8000) {
 		warnx("Overlay is not exactly 0x8000 bytes large");
 	} else if (overlaySize < 0x8000) {
 		warnx("Overlay is less than 0x8000 bytes large");
@@ -136,7 +138,7 @@ static uint32_t checkOverlaySize() {
 // This ensures that `writeROM` will output each bank, even if some are not
 // covered by any sections.
 static void coverOverlayBanks(uint32_t nbOverlayBanks) {
-	// 2 if is32kMode, 1 otherwise
+	// 2 if options.is32kMode, 1 otherwise
 	uint32_t nbRom0Banks = sectionTypeInfo[SECTTYPE_ROM0].size / BANK_SIZE;
 	// Discount ROM0 banks to avoid outputting too much
 	uint32_t nbUncoveredBanks = nbOverlayBanks - nbRom0Banks > sections[SECTTYPE_ROMX].size()
@@ -144,7 +146,7 @@ static void coverOverlayBanks(uint32_t nbOverlayBanks) {
 	                                : 0;
 
 	if (nbUncoveredBanks > sections[SECTTYPE_ROMX].size()) {
-		for (uint32_t i = sections[SECTTYPE_ROMX].size(); i < nbUncoveredBanks; i++) {
+		for (uint32_t i = sections[SECTTYPE_ROMX].size(); i < nbUncoveredBanks; ++i) {
 			sections[SECTTYPE_ROMX].emplace_back();
 		}
 	}
@@ -157,16 +159,15 @@ static uint8_t getNextFillByte() {
 			return c;
 		}
 
-		if (static bool warned = false; !hasPadValue && !warned) {
+		if (static bool warned = false; !options.hasPadValue && !warned) {
 			warnx("Output is larger than overlay file, but no padding value was specified");
 			warned = true;
 		}
 	}
 
-	return padValue;
+	return options.padValue;
 }
 
-// Write a ROM bank's sections, ordered by increasing address, to the output file.
 static void
     writeBank(std::deque<Section const *> *bankSections, uint16_t baseOffset, uint16_t size) {
 	uint16_t offset = 0;
@@ -177,41 +178,42 @@ static void
 			// Output padding up to the next SECTION
 			while (offset + baseOffset < section->org) {
 				putc(getNextFillByte(), outputFile);
-				offset++;
+				++offset;
 			}
 
 			// Output the section itself
 			fwrite(section->data.data(), 1, section->size, outputFile);
-			if (overlayFile) {
-				// Skip bytes even with pipes
-				for (uint16_t i = 0; i < section->size; i++) {
-					getc(overlayFile);
-				}
-			}
 			offset += section->size;
+
+			if (!overlayFile) {
+				continue;
+			}
+			// Skip bytes even with pipes
+			for (uint16_t i = 0; i < section->size; ++i) {
+				getc(overlayFile);
+			}
 		}
 	}
 
-	if (!disablePadding) {
+	if (!options.disablePadding) {
 		while (offset < size) {
 			putc(getNextFillByte(), outputFile);
-			offset++;
+			++offset;
 		}
 	}
 }
 
-// Writes a ROM file to the output.
 static void writeROM() {
-	if (outputFileName) {
-		if (strcmp(outputFileName, "-")) {
-			outputFile = fopen(outputFileName, "wb");
+	if (options.outputFileName) {
+		if (strcmp(options.outputFileName, "-")) {
+			outputFile = fopen(options.outputFileName, "wb");
 		} else {
-			outputFileName = "<stdout>";
+			options.outputFileName = "<stdout>";
 			(void)setmode(STDOUT_FILENO, O_BINARY);
 			outputFile = stdout;
 		}
 		if (!outputFile) {
-			err("Failed to open output file \"%s\"", outputFileName);
+			fatal("Failed to open output file \"%s\": %s", options.outputFileName, strerror(errno));
 		}
 	}
 	Defer closeOutputFile{[&] {
@@ -220,16 +222,18 @@ static void writeROM() {
 		}
 	}};
 
-	if (overlayFileName) {
-		if (strcmp(overlayFileName, "-")) {
-			overlayFile = fopen(overlayFileName, "rb");
+	if (options.overlayFileName) {
+		if (strcmp(options.overlayFileName, "-")) {
+			overlayFile = fopen(options.overlayFileName, "rb");
 		} else {
-			overlayFileName = "<stdin>";
+			options.overlayFileName = "<stdin>";
 			(void)setmode(STDIN_FILENO, O_BINARY);
 			overlayFile = stdin;
 		}
 		if (!overlayFile) {
-			err("Failed to open overlay file \"%s\"", overlayFileName);
+			fatal(
+			    "Failed to open overlay file \"%s\": %s", options.overlayFileName, strerror(errno)
+			);
 		}
 	}
 	Defer closeOverlayFile{[&] {
@@ -251,7 +255,7 @@ static void writeROM() {
 		    sectionTypeInfo[SECTTYPE_ROM0].size
 		);
 
-		for (uint32_t i = 0; i < sections[SECTTYPE_ROMX].size(); i++) {
+		for (uint32_t i = 0; i < sections[SECTTYPE_ROMX].size(); ++i) {
 			writeBank(
 			    &sections[SECTTYPE_ROMX][i].sections,
 			    sectionTypeInfo[SECTTYPE_ROMX].startAddr,
@@ -261,87 +265,72 @@ static void writeROM() {
 	}
 }
 
-// Prints a symbol's name to a file, assuming that the first character is legal.
-// Illegal characters are UTF-8-decoded (errors are replaced by U+FFFD) and emitted as '\u'/'\U'.
-static void printSymName(std::string const &name, FILE *file) {
+static void writeSymName(std::string const &name, FILE *file) {
 	for (char const *ptr = name.c_str(); *ptr != '\0';) {
-		char c = *ptr;
-
-		if (continuesIdentifier(c)) {
-			// Output legal ASCII characters as-is
+		// Output legal ASCII characters as-is
+		if (char c = *ptr; continuesIdentifier(c)) {
 			putc(c, file);
 			++ptr;
-		} else {
-			// Output illegal characters using Unicode escapes
-			// Decode the UTF-8 codepoint; or at least attempt to
-			uint32_t state = 0, codepoint;
-
-			do {
-				decode(&state, &codepoint, *ptr);
-				if (state == 1) {
-					// This sequence was invalid; emit a U+FFFD, and recover
-					codepoint = 0xFFFD;
-					// Skip continuation bytes
-					// A NUL byte does not qualify, so we're good
-					while ((*ptr & 0xC0) == 0x80) {
-						++ptr;
-					}
-					break;
-				}
-				++ptr;
-			} while (state != 0);
-
-			fprintf(file, codepoint <= 0xFFFF ? "\\u%04" PRIx32 : "\\U%08" PRIx32, codepoint);
+			continue;
 		}
+
+		// Output illegal characters using Unicode escapes ('\u' or '\U')
+		// Decode the UTF-8 codepoint; or at least attempt to
+		uint32_t state = UTF8_ACCEPT, codepoint;
+		do {
+			decode(&state, &codepoint, *ptr);
+			if (state != UTF8_REJECT) {
+				++ptr;
+				continue;
+			}
+			// This sequence was invalid; emit a U+FFFD, and recover
+			codepoint = 0xFFFD;
+			// Skip continuation bytes
+			// A NUL byte does not qualify, so we're good
+			while ((*ptr & 0xC0) == 0x80) {
+				++ptr;
+			}
+			break;
+		} while (state != UTF8_ACCEPT);
+		fprintf(file, codepoint <= 0xFFFF ? "\\u%04" PRIx32 : "\\U%08" PRIx32, codepoint);
 	}
 }
 
 // Comparator function for `std::stable_sort` to sort symbols
 static bool compareSymbols(SortedSymbol const &sym1, SortedSymbol const &sym2) {
-	// First, sort by address
-	if (sym1.addr != sym2.addr) {
-		return sym1.addr < sym2.addr;
-	}
-
-	// Second, sort by locality (global before local)
 	std::string const &sym1_name = sym1.sym->name;
 	std::string const &sym2_name = sym2.sym->name;
 	bool sym1_local = sym1_name.find('.') != std::string::npos;
 	bool sym2_local = sym2_name.find('.') != std::string::npos;
-	if (sym1_local != sym2_local) {
-		return sym1_local < sym2_local;
-	}
 
+	// First, sort by address
+	// Second, sort by locality (global before local)
 	// Third, sort by parent address
-	if (sym1.parentAddr != sym2.parentAddr) {
-		return sym1.parentAddr < sym2.parentAddr;
-	}
-
 	// Fourth, sort by name
-	return sym1_name < sym2_name;
+	return std::tie(sym1.addr, sym1_local, sym1.parentAddr, sym1_name)
+	       < std::tie(sym2.addr, sym2_local, sym2.parentAddr, sym2_name);
 }
 
-// Write a bank's contents to the sym file
-static void writeSymBank(SortedSections const &bankSections, SectionType type, uint32_t bank) {
-#define forEachSortedSection(sect, ...) \
-	do { \
-		for (auto it = bankSections.zeroLenSections.begin(); \
-		     it != bankSections.zeroLenSections.end(); \
-		     it++) { \
-			for (Section const *sect = *it; sect; sect = sect->nextu.get()) { \
-				__VA_ARGS__ \
-			} \
-		} \
-		for (auto it = bankSections.sections.begin(); it != bankSections.sections.end(); it++) { \
-			for (Section const *sect = *it; sect; sect = sect->nextu.get()) { \
-				__VA_ARGS__ \
-			} \
-		} \
-	} while (0)
+template<typename F>
+static void forEachSortedSection(SortedSections const &bankSections, F callback) {
+	for (Section const *sect : bankSections.zeroLenSections) {
+		for (; sect; sect = sect->nextu.get()) {
+			callback(*sect);
+		}
+	}
+	for (Section const *sect : bankSections.sections) {
+		for (; sect; sect = sect->nextu.get()) {
+			callback(*sect);
+		}
+	}
+}
 
+static void writeSymBank(SortedSections const &bankSections, SectionType type, uint32_t bank) {
 	uint32_t nbSymbols = 0;
 
-	forEachSortedSection(sect, { nbSymbols += sect->symbols.size(); });
+	forEachSortedSection(bankSections, [&](Section const &sect) {
+		nbSymbols += sect.symbols.size();
+	});
 
 	if (!nbSymbols) {
 		return;
@@ -351,36 +340,34 @@ static void writeSymBank(SortedSections const &bankSections, SectionType type, u
 
 	symList.reserve(nbSymbols);
 
-	forEachSortedSection(sect, {
-		for (Symbol const *sym : sect->symbols) {
+	forEachSortedSection(bankSections, [&symList](Section const &sect) {
+		for (Symbol const *sym : sect.symbols) {
 			// Don't output symbols that begin with an illegal character
-			if (!sym->name.empty() && startsIdentifier(sym->name[0])) {
-				uint16_t addr = static_cast<uint16_t>(sym->label().offset + sect->org);
-				uint16_t parentAddr = addr;
-				if (auto pos = sym->name.find('.'); pos != std::string::npos) {
-					std::string parentName = sym->name.substr(0, pos);
-					if (Symbol const *parentSym = sym_GetSymbol(parentName);
-					    parentSym && parentSym->data.holds<Label>()) {
-						auto const &parentLabel = parentSym->label();
-						assume(parentLabel.section != nullptr);
-						parentAddr =
-						    static_cast<uint16_t>(parentLabel.offset + parentLabel.section->org);
-					}
-				}
-				symList.push_back({.sym = sym, .addr = addr, .parentAddr = parentAddr});
+			if (sym->name.empty() || !startsIdentifier(sym->name[0])) {
+				continue;
 			}
+			uint16_t addr = static_cast<uint16_t>(sym->label().offset + sect.org);
+			uint16_t parentAddr = addr;
+			if (auto pos = sym->name.find('.'); pos != std::string::npos) {
+				std::string parentName = sym->name.substr(0, pos);
+				if (Symbol const *parentSym = sym_GetSymbol(parentName);
+				    parentSym && std::holds_alternative<Label>(parentSym->data)) {
+					Label const &parentLabel = parentSym->label();
+					Section const &parentSection = *parentLabel.section;
+					parentAddr = static_cast<uint16_t>(parentLabel.offset + parentSection.org);
+				}
+			}
+			symList.push_back({.sym = sym, .addr = addr, .parentAddr = parentAddr});
 		}
 	});
 
-#undef forEachSortedSection
-
 	std::stable_sort(RANGE(symList), compareSymbols);
 
 	uint32_t symBank = bank + sectionTypeInfo[type].firstBank;
 
 	for (SortedSymbol &sym : symList) {
 		fprintf(symFile, "%02" PRIx32 ":%04" PRIx16 " ", symBank, sym.addr);
-		printSymName(sym.sym->name, symFile);
+		writeSymName(sym.sym->name, symFile);
 		putc('\n', symFile);
 	}
 }
@@ -400,8 +387,7 @@ static void writeEmptySpace(uint16_t begin, uint16_t end) {
 	}
 }
 
-// Prints a section's name to a file.
-static void printSectionName(std::string const &name, FILE *file) {
+static void writeSectionName(std::string const &name, FILE *file) {
 	for (char c : name) {
 		// Escape characters that need escaping
 		switch (c) {
@@ -425,7 +411,47 @@ static void printSectionName(std::string const &name, FILE *file) {
 	}
 }
 
-// Write a bank's contents to the map file
+template<typename F>
+uint16_t forEachSection(SortedSections const &sectList, F callback) {
+	uint16_t used = 0;
+	auto section = sectList.sections.begin();
+	auto zeroLenSection = sectList.zeroLenSections.begin();
+	while (section != sectList.sections.end() || zeroLenSection != sectList.zeroLenSections.end()) {
+		// Pick the lowest section by address out of the two
+		auto &pickedSection = section == sectList.sections.end()                 ? zeroLenSection
+		                      : zeroLenSection == sectList.zeroLenSections.end() ? section
+		                      : (*section)->org < (*zeroLenSection)->org         ? section
+		                                                                         : zeroLenSection;
+		used += (*pickedSection)->size;
+		callback(**pickedSection);
+		++pickedSection;
+	}
+	return used;
+}
+
+static void writeMapSymbols(Section const *sect) {
+	for (uint16_t org = sect->org; sect; sect = sect->nextu.get()) {
+		for (Symbol *sym : sect->symbols) {
+			// Don't output symbols that begin with an illegal character
+			if (sym->name.empty() || !startsIdentifier(sym->name[0])) {
+				continue;
+			}
+			// Space matches "\tSECTION: $xxxx ..."
+			fprintf(mapFile, "\t         $%04" PRIx32 " = ", sym->label().offset + org);
+			writeSymName(sym->name, mapFile);
+			putc('\n', mapFile);
+		}
+
+		// Announce the following "piece"
+		if (SectionModifier mod = sect->nextu ? sect->nextu->modifier : SECTION_NORMAL;
+		    mod == SECTION_UNION) {
+			fputs("\t         ; Next union\n", mapFile);
+		} else if (mod == SECTION_FRAGMENT) {
+			fputs("\t         ; Next fragment\n", mapFile);
+		}
+	}
+}
+
 static void writeMapBank(SortedSections const &sectList, SectionType type, uint32_t bank) {
 	fprintf(
 	    mapFile,
@@ -434,60 +460,27 @@ static void writeMapBank(SortedSections const &sectList, SectionType type, uint3
 	    bank + sectionTypeInfo[type].firstBank
 	);
 
-	uint16_t used = 0;
-	auto section = sectList.sections.begin();
-	auto zeroLenSection = sectList.zeroLenSections.begin();
 	uint16_t prevEndAddr = sectionTypeInfo[type].startAddr;
+	uint16_t used = forEachSection(sectList, [&](Section const &sect) {
+		assume(sect.offset == 0);
 
-	while (section != sectList.sections.end() || zeroLenSection != sectList.zeroLenSections.end()) {
-		// Pick the lowest section by address out of the two
-		auto &pickedSection = section == sectList.sections.end()                 ? zeroLenSection
-		                      : zeroLenSection == sectList.zeroLenSections.end() ? section
-		                      : (*section)->org < (*zeroLenSection)->org         ? section
-		                                                                         : zeroLenSection;
-		Section const *sect = *pickedSection;
+		writeEmptySpace(prevEndAddr, sect.org);
 
-		used += sect->size;
-		assume(sect->offset == 0);
+		prevEndAddr = sect.org + sect.size;
 
-		writeEmptySpace(prevEndAddr, sect->org);
-
-		prevEndAddr = sect->org + sect->size;
-
-		fprintf(mapFile, "\tSECTION: $%04" PRIx16, sect->org);
-		if (sect->size != 0) {
+		fprintf(mapFile, "\tSECTION: $%04" PRIx16, sect.org);
+		if (sect.size != 0) {
 			fprintf(mapFile, "-$%04x", prevEndAddr - 1);
 		}
-		fprintf(mapFile, " ($%04" PRIx16 " byte%s) [\"", sect->size, sect->size == 1 ? "" : "s");
-		printSectionName(sect->name, mapFile);
+		fprintf(mapFile, " ($%04" PRIx16 " byte%s) [\"", sect.size, sect.size == 1 ? "" : "s");
+		writeSectionName(sect.name, mapFile);
 		fputs("\"]\n", mapFile);
 
-		if (!noSymInMap) {
+		if (!options.noSymInMap) {
 			// Also print symbols in the following "pieces"
-			for (uint16_t org = sect->org; sect; sect = sect->nextu.get()) {
-				for (Symbol *sym : sect->symbols) {
-					// Don't output symbols that begin with an illegal character
-					if (!sym->name.empty() && startsIdentifier(sym->name[0])) {
-						// Space matches "\tSECTION: $xxxx ..."
-						fprintf(mapFile, "\t         $%04" PRIx32 " = ", sym->label().offset + org);
-						printSymName(sym->name, mapFile);
-						putc('\n', mapFile);
-					}
-				}
-
-				if (sect->nextu) {
-					// Announce the following "piece"
-					if (sect->nextu->modifier == SECTION_UNION) {
-						fprintf(mapFile, "\t         ; Next union\n");
-					} else if (sect->nextu->modifier == SECTION_FRAGMENT) {
-						fprintf(mapFile, "\t         ; Next fragment\n");
-					}
-				}
-			}
+			writeMapSymbols(&sect);
 		}
-
-		pickedSection++;
-	}
+	});
 
 	if (used == 0) {
 		fputs("\tEMPTY\n", mapFile);
@@ -502,11 +495,10 @@ static void writeMapBank(SortedSections const &sectList, SectionType type, uint3
 	}
 }
 
-// Write the total used and free space by section type to the map file
 static void writeMapSummary() {
 	fputs("SUMMARY:\n", mapFile);
 
-	for (uint8_t i = 0; i < SECTTYPE_INVALID; i++) {
+	for (uint8_t i = 0; i < SECTTYPE_INVALID; ++i) {
 		SectionType type = typeMap[i];
 		uint32_t nbBanks = sections[type].size();
 
@@ -522,25 +514,8 @@ static void writeMapSummary() {
 
 		uint32_t usedTotal = 0;
 
-		for (uint32_t bank = 0; bank < nbBanks; bank++) {
-			uint16_t used = 0;
-			auto &sectList = sections[type][bank];
-			auto section = sectList.sections.begin();
-			auto zeroLenSection = sectList.zeroLenSections.begin();
-
-			while (section != sectList.sections.end()
-			       || zeroLenSection != sectList.zeroLenSections.end()) {
-				// Pick the lowest section by address out of the two
-				auto &pickedSection = section == sectList.sections.end() ? zeroLenSection
-				                      : zeroLenSection == sectList.zeroLenSections.end() ? section
-				                      : (*section)->org < (*zeroLenSection)->org         ? section
-				                                                                 : zeroLenSection;
-
-				used += (*pickedSection)->size;
-				pickedSection++;
-			}
-
-			usedTotal += used;
+		for (uint32_t bank = 0; bank < nbBanks; ++bank) {
+			usedTotal += forEachSection(sections[type][bank], [](Section const &) {});
 		}
 
 		fprintf(
@@ -558,30 +533,29 @@ static void writeMapSummary() {
 	}
 }
 
-// Writes the sym file, if applicable.
 static void writeSym() {
-	if (!symFileName) {
+	if (!options.symFileName) {
 		return;
 	}
 
-	if (strcmp(symFileName, "-")) {
-		symFile = fopen(symFileName, "w");
+	if (strcmp(options.symFileName, "-")) {
+		symFile = fopen(options.symFileName, "w");
 	} else {
-		symFileName = "<stdout>";
+		options.symFileName = "<stdout>";
 		(void)setmode(STDOUT_FILENO, O_TEXT); // May have been set to O_BINARY previously
 		symFile = stdout;
 	}
 	if (!symFile) {
-		err("Failed to open sym file \"%s\"", symFileName);
+		fatal("Failed to open sym file \"%s\": %s", options.symFileName, strerror(errno));
 	}
 	Defer closeSymFile{[&] { fclose(symFile); }};
 
 	fputs("; File generated by rgblink\n", symFile);
 
-	for (uint8_t i = 0; i < SECTTYPE_INVALID; i++) {
+	for (uint8_t i = 0; i < SECTTYPE_INVALID; ++i) {
 		SectionType type = typeMap[i];
 
-		for (uint32_t bank = 0; bank < sections[type].size(); bank++) {
+		for (uint32_t bank = 0; bank < sections[type].size(); ++bank) {
 			writeSymBank(sections[type][bank], type, bank);
 		}
 	}
@@ -591,48 +565,47 @@ static void writeSym() {
 	constants.clear();
 	sym_ForEach([](Symbol &sym) {
 		// Symbols are already limited to the exported ones
-		if (sym.data.holds<int32_t>()) {
+		if (std::holds_alternative<int32_t>(sym.data)) {
 			constants.push_back(&sym);
 		}
 	});
 	// Numeric constants are ordered by value, then by name
 	std::sort(RANGE(constants), [](Symbol *sym1, Symbol *sym2) -> bool {
-		int32_t val1 = sym1->data.get<int32_t>(), val2 = sym2->data.get<int32_t>();
-		return val1 != val2 ? val1 < val2 : sym1->name < sym2->name;
+		return std::tie(std::get<int32_t>(sym1->data), sym1->name)
+		       < std::tie(std::get<int32_t>(sym2->data), sym2->name);
 	});
 	for (Symbol *sym : constants) {
-		int32_t val = sym->data.get<int32_t>();
+		int32_t val = std::get<int32_t>(sym->data);
 		int width = val < 0x100 ? 2 : val < 0x10000 ? 4 : 8;
 		fprintf(symFile, "%0*" PRIx32 " ", width, val);
-		printSymName(sym->name, symFile);
+		writeSymName(sym->name, symFile);
 		putc('\n', symFile);
 	}
 }
 
-// Writes the map file, if applicable.
 static void writeMap() {
-	if (!mapFileName) {
+	if (!options.mapFileName) {
 		return;
 	}
 
-	if (strcmp(mapFileName, "-")) {
-		mapFile = fopen(mapFileName, "w");
+	if (strcmp(options.mapFileName, "-")) {
+		mapFile = fopen(options.mapFileName, "w");
 	} else {
-		mapFileName = "<stdout>";
+		options.mapFileName = "<stdout>";
 		(void)setmode(STDOUT_FILENO, O_TEXT); // May have been set to O_BINARY previously
 		mapFile = stdout;
 	}
 	if (!mapFile) {
-		err("Failed to open map file \"%s\"", mapFileName);
+		fatal("Failed to open map file \"%s\": %s", options.mapFileName, strerror(errno));
 	}
 	Defer closeMapFile{[&] { fclose(mapFile); }};
 
 	writeMapSummary();
 
-	for (uint8_t i = 0; i < SECTTYPE_INVALID; i++) {
+	for (uint8_t i = 0; i < SECTTYPE_INVALID; ++i) {
 		SectionType type = typeMap[i];
 
-		for (uint32_t bank = 0; bank < sections[type].size(); bank++) {
+		for (uint32_t bank = 0; bank < sections[type].size(); ++bank) {
 			writeMapBank(sections[type][bank], type, bank);
 		}
 	}
