@@ -2,12 +2,14 @@
 
 #include "util.hpp"
 
+#include <algorithm>
 #include <ctype.h>
 #include <stdint.h>
 #include <stdio.h>
 
 bool startsIdentifier(int c) {
-	// This returns false for anonymous labels, which internally start with a '!'
+	// This returns false for anonymous labels, which internally start with a '!',
+	// and for section fragment literal labels, which internally start with a '$'.
 	return (c >= 'A' && c <= 'Z') || (c >= 'a' && c <= 'z') || c == '.' || c == '_';
 }
 
@@ -59,3 +61,19 @@ char const *printChar(int c) {
 	buf[4] = '\0';
 	return buf;
 }
+
+// FNV-1a hash of an uppercased string
+size_t Uppercase::operator()(std::string const &str) const {
+	size_t hash = 0x811C9DC5;
+	for (char const &c : str) {
+		hash = (hash ^ toupper(c)) * 16777619;
+	}
+	return hash;
+}
+
+// Compare two strings without case-sensitivity (by converting to uppercase)
+bool Uppercase::operator()(std::string const &str1, std::string const &str2) const {
+	return std::equal(RANGE(str1), RANGE(str2), [](char c1, char c2) {
+		return toupper(c1) == toupper(c2);
+	});
+}
